@@ -18,7 +18,7 @@
     Boston, MA 02111-1307, USA.
         */
 
-// $Id: kapplication.cpp,v 1.637.2.12 2004/05/29 13:42:37 binner Exp $
+// $Id: kapplication.cpp,v 1.670.2.1 2004/08/17 13:44:00 waba Exp $
 
 #include "config.h"
 
@@ -75,7 +75,7 @@
 #include <kprotocolinfo.h>
 
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
-#include <kstartupinfo.h> // schroder
+#include <kstartupinfo.h>
 #endif
 
 #include <dcopclient.h>
@@ -100,7 +100,7 @@
 #include <netdb.h>
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
 //#ifndef Q_WS_QWS //FIXME(E): NetWM should talk to QWS...
-#include <netwm.h> // schroder
+#include <netwm.h>
 #endif
 
 #include "kprocctrl.h"
@@ -126,13 +126,14 @@
 #endif
 
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
-#include <kipc.h> // schroder
+#include <kipc.h>
 #endif
 
 #include "kappdcopiface.h"
 
 bool kde_have_kipc = true; // magic hook to disable kipc in kdm
 bool kde_kiosk_exception = false; // flag to disable kiosk restrictions
+bool kde_kiosk_admin = false;
 
 KApplication* KApplication::KApp = 0L;
 bool KApplication::loadedByKdeinit = false;
@@ -141,18 +142,8 @@ bool KApplication::s_dcopClientNeedsPostInit = false;
 
 static Atom atom_DesktopWindow;
 static Atom atom_NetSupported;
-#if KDE_IS_VERSION( 3, 2, 91 )
-#warning Obsolete, remove.
-// remove atom_KdeNetUserTime related stuff (l.lunak@kde.org)
-#endif
-static Atom atom_KdeNetUserTime;
-static Atom kde_net_wm_user_time     = 0;
-#if KDE_IS_VERSION( 3, 2, 91 )
-#warning This should be in Qt already, check.
-// remove things related to qt_x_user_time that should be in Qt by now (l.lunak@kde.org)
-#endif
-Time   qt_x_user_time = CurrentTime;
 extern Time qt_x_time;
+extern Time qt_x_user_time;
 static Atom kde_xdnd_drop;
 
 // duplicated from patched Qt, so that there won't be unresolved symbols if Qt gets
@@ -428,9 +419,9 @@ bool KApplication::notify(QObject *receiver, QEvent *event)
     if ((t == QEvent::AccelOverride) || (t == QEvent::KeyPress))
     {
        static const KShortcut& _selectAll = KStdAccel::selectAll();
-       if (receiver && receiver->inherits("QLineEdit"))
+       QLineEdit *edit = ::qt_cast<QLineEdit *>(receiver);
+       if (edit)
        {
-          QLineEdit *edit = static_cast<QLineEdit *>(receiver);
           // We have a keypress for a lineedit...
           QKeyEvent *kevent = static_cast<QKeyEvent *>(event);
           KKey key(kevent);
@@ -466,9 +457,9 @@ bool KApplication::notify(QObject *receiver, QEvent *event)
 
           }
        }
-       if (receiver && receiver->inherits("QTextEdit"))
+       QTextEdit *medit = ::qt_cast<QTextEdit *>(receiver);
+       if (medit)
        {
-          QTextEdit *medit = static_cast<QTextEdit *>(receiver);
           // We have a keypress for a multilineedit...
           QKeyEvent *kevent = static_cast<QKeyEvent *>(event);
           if (_selectAll.contains(KKey(kevent)))
@@ -485,18 +476,11 @@ bool KApplication::notify(QObject *receiver, QEvent *event)
           }
        }
     }
-    if( event->type() == QEvent::Show && receiver->isWidgetType())
+    if( t == QEvent::Show && receiver->isWidgetType())
     {
 	QWidget* w = static_cast< QWidget* >( receiver );
         if( w->isTopLevel() && !startupId().isEmpty()) // TODO better done using window group leader?
             KStartupInfo::setWindowStartupId( w->winId(), startupId());
-	if( w->isTopLevel() && qt_x_user_time != CurrentTime ) // CurrentTime means no input event yet
-            XChangeProperty( qt_xdisplay(), w->winId(), kde_net_wm_user_time, XA_CARDINAL,
-                32, PropModeReplace, (unsigned char*)&qt_x_user_time, 1 );
-    }
-    if( event->type() == QEvent::Show && receiver->isWidgetType())
-    {
-	QWidget* w = static_cast< QWidget* >( receiver );
         if( w->isTopLevel() && !w->testWFlags( WX11BypassWM ) && !w->isPopup() && !event->spontaneous())
         {
             if( d->app_started_timer == NULL )
@@ -594,6 +578,38 @@ KApplication::KApplication( bool allowStyles, bool GUIenabled ) :
     d->m_KAppDCOPInterface = new KAppDCOPInterface(this);
 }
 
+#ifdef Q_WS_X11
+KApplication::KApplication( Display *dpy, Qt::HANDLE visual, Qt::HANDLE colormap,
+		            bool allowStyles ) :
+  QApplication( dpy, *KCmdLineArgs::qt_argc(), *KCmdLineArgs::qt_argv(),
+                visual, colormap ),
+  KInstance( KCmdLineArgs::about), display(0L), d (new KApplicationPrivate)
+{
+    read_app_startup_id();
+    useStyles = allowStyles;
+    setName( instanceName() );
+    installSigpipeHandler();
+    parseCommandLine( );
+    init( true );
+    d->m_KAppDCOPInterface = new KAppDCOPInterface(this);
+}
+
+KApplication::KApplication( Display *dpy, Qt::HANDLE visual, Qt::HANDLE colormap,
+		            bool allowStyles, KInstance * _instance ) :
+  QApplication( dpy, *KCmdLineArgs::qt_argc(), *KCmdLineArgs::qt_argv(),
+                visual, colormap ),
+  KInstance( _instance ), display(0L), d (new KApplicationPrivate)
+{
+    read_app_startup_id();
+    useStyles = allowStyles;
+    setName( instanceName() );
+    installSigpipeHandler();
+    parseCommandLine( );
+    init( true );
+    d->m_KAppDCOPInterface = new KAppDCOPInterface(this);
+}
+#endif
+
 KApplication::KApplication( bool allowStyles, bool GUIenabled, KInstance* _instance ) :
   QApplication( *KCmdLineArgs::qt_argc(), *KCmdLineArgs::qt_argv(),
                 GUIenabled ),
@@ -721,12 +737,6 @@ void KApplication::init(bool GUIenabled)
       atoms[n] = &atom_NetSupported;
       names[n++] = (char *) "_NET_SUPPORTED";
 
-      atoms[n] = &atom_KdeNetUserTime;
-      names[n++] = (char *) "_KDE_NET_USER_TIME";
-
-      atoms[n] = &kde_net_wm_user_time;
-      names[n++] = (char *) "_NET_WM_USER_TIME";
-      
       atoms[n] = &kde_xdnd_drop;
       names[n++] = (char *) "XdndDrop";
 
@@ -1260,8 +1270,8 @@ void KApplication::dcopFailure(const QString &msg)
   }
   if (failureCount == 2)
   {
-     QString msgStr(i18n("There was an error setting up inter-process\n"
-                      "communications for KDE. The message returned\n"
+     QString msgStr(i18n("There was an error setting up inter-process "
+                      "communications for KDE. The message returned "
                       "by the system was:\n\n"));
      msgStr += msg;
      msgStr += i18n("\n\nPlease check that the \"dcopserver\" program is running!");
@@ -1273,7 +1283,7 @@ void KApplication::dcopFailure(const QString &msg)
            kapp->mainWidget(),
            i18n("DCOP communications error (%1)").arg(kapp->caption()),
            msgStr,
-           i18n("OK")
+           i18n("&OK")
          );
      }
      else
@@ -1289,50 +1299,50 @@ static const KCmdLineOptions qt_options[] =
 {
   //FIXME: Check if other options are specific to Qt/X11
 #ifdef Q_WS_X11
-   { "display <displayname>", I18N_NOOP("Use the X-server display 'displayname'."), 0},
+   { "display <displayname>", I18N_NOOP("Use the X-server display 'displayname'"), 0},
 #else
-   { "display <displayname>", I18N_NOOP("Use the QWS display 'displayname'."), 0},
+   { "display <displayname>", I18N_NOOP("Use the QWS display 'displayname'"), 0},
 #endif
-   { "session <sessionId>", I18N_NOOP("Restore the application for the given 'sessionId'."), 0},
-   { "cmap", I18N_NOOP("Causes the application to install a private color\nmap on an 8-bit display."), 0},
-   { "ncols <count>", I18N_NOOP("Limits the number of colors allocated in the color\ncube on an 8-bit display, if the application is\nusing the QApplication::ManyColor color\nspecification."), 0},
-   { "nograb", I18N_NOOP("tells Qt to never grab the mouse or the keyboard."), 0},
-   { "dograb", I18N_NOOP("running under a debugger can cause an implicit\n-nograb, use -dograb to override."), 0},
-   { "sync", I18N_NOOP("switches to synchronous mode for debugging."), 0},
+   { "session <sessionId>", I18N_NOOP("Restore the application for the given 'sessionId'"), 0},
+   { "cmap", I18N_NOOP("Causes the application to install a private color\nmap on an 8-bit display"), 0},
+   { "ncols <count>", I18N_NOOP("Limits the number of colors allocated in the color\ncube on an 8-bit display, if the application is\nusing the QApplication::ManyColor color\nspecification"), 0},
+   { "nograb", I18N_NOOP("tells Qt to never grab the mouse or the keyboard"), 0},
+   { "dograb", I18N_NOOP("running under a debugger can cause an implicit\n-nograb, use -dograb to override"), 0},
+   { "sync", I18N_NOOP("switches to synchronous mode for debugging"), 0},
    { "fn", 0, 0},
-   { "font <fontname>", I18N_NOOP("defines the application font."), 0},
+   { "font <fontname>", I18N_NOOP("defines the application font"), 0},
    { "bg", 0, 0},
-   { "background <color>", I18N_NOOP("sets the default background color and an\napplication palette (light and dark shades are\ncalculated)."), 0},
+   { "background <color>", I18N_NOOP("sets the default background color and an\napplication palette (light and dark shades are\ncalculated)"), 0},
    { "fg", 0, 0},
-   { "foreground <color>", I18N_NOOP("sets the default foreground color."), 0},
+   { "foreground <color>", I18N_NOOP("sets the default foreground color"), 0},
    { "btn", 0, 0},
-   { "button <color>", I18N_NOOP("sets the default button color."), 0},
-   { "name <name>", I18N_NOOP("sets the application name."), 0},
-   { "title <title>", I18N_NOOP("sets the application title (caption)."), 0},
+   { "button <color>", I18N_NOOP("sets the default button color"), 0},
+   { "name <name>", I18N_NOOP("sets the application name"), 0},
+   { "title <title>", I18N_NOOP("sets the application title (caption)"), 0},
 #ifdef Q_WS_X11
-   { "visual TrueColor", I18N_NOOP("forces the application to use a TrueColor visual on\nan 8-bit display."), 0},
-   { "inputstyle <inputstyle>", I18N_NOOP("sets XIM (X Input Method) input style. Possible\nvalues are onthespot, overthespot, offthespot and\nroot."), 0 },
-   { "im <XIM server>", I18N_NOOP("set XIM server."),0},
-   { "noxim", I18N_NOOP("disable XIM."), 0 },
+   { "visual TrueColor", I18N_NOOP("forces the application to use a TrueColor visual on\nan 8-bit display"), 0},
+   { "inputstyle <inputstyle>", I18N_NOOP("sets XIM (X Input Method) input style. Possible\nvalues are onthespot, overthespot, offthespot and\nroot"), 0 },
+   { "im <XIM server>", I18N_NOOP("set XIM server"),0},
+   { "noxim", I18N_NOOP("disable XIM"), 0 },
 #endif
 #ifdef Q_WS_QWS
-   { "qws", I18N_NOOP("forces the application to run as QWS Server."), 0},
+   { "qws", I18N_NOOP("forces the application to run as QWS Server"), 0},
 #endif
-   { "reverse", I18N_NOOP("mirrors the whole layout of widgets."), 0},
+   { "reverse", I18N_NOOP("mirrors the whole layout of widgets"), 0},
    KCmdLineLastOption
 };
 
 static const KCmdLineOptions kde_options[] =
 {
-   { "caption <caption>",       I18N_NOOP("Use 'caption' as name in the titlebar."), 0},
-   { "icon <icon>",             I18N_NOOP("Use 'icon' as the application icon."), 0},
-   { "miniicon <icon>",         I18N_NOOP("Use 'icon' as the icon in the titlebar."), 0},
-   { "config <filename>",       I18N_NOOP("Use alternative configuration file."), 0},
-   { "dcopserver <server>",     I18N_NOOP("Use the DCOP Server specified by 'server'."), 0},
-   { "nocrashhandler",          I18N_NOOP("Disable crash handler, to get core dumps."), 0},
-   { "waitforwm",          I18N_NOOP("Waits for a WM_NET compatible windowmanager."), 0},
-   { "style <style>", I18N_NOOP("sets the application GUI style."), 0},
-   { "geometry <geometry>", I18N_NOOP("sets the client geometry of the main widget."), 0},
+   { "caption <caption>",       I18N_NOOP("Use 'caption' as name in the titlebar"), 0},
+   { "icon <icon>",             I18N_NOOP("Use 'icon' as the application icon"), 0},
+   { "miniicon <icon>",         I18N_NOOP("Use 'icon' as the icon in the titlebar"), 0},
+   { "config <filename>",       I18N_NOOP("Use alternative configuration file"), 0},
+   { "dcopserver <server>",     I18N_NOOP("Use the DCOP Server specified by 'server'"), 0},
+   { "nocrashhandler",          I18N_NOOP("Disable crash handler, to get core dumps"), 0},
+   { "waitforwm",          I18N_NOOP("Waits for a WM_NET compatible windowmanager"), 0},
+   { "style <style>", I18N_NOOP("sets the application GUI style"), 0},
+   { "geometry <geometry>", I18N_NOOP("sets the client geometry of the main widget"), 0},
 #if QT_VERSION < 0x030100
    { "smkey <sessionKey>", I18N_NOOP("Define a 'sessionKey' for the session id. Only valid with -session"), 0},
 #else
@@ -1547,29 +1557,9 @@ void KApplication::dcopBlockUserInput( bool b )
 bool KApplication::x11EventFilter( XEvent *_event )
 {
     switch ( _event->type ) {
-        case ButtonPress:
-        case XKeyPress:
-        {
-	    if( _event->type == ButtonPress )
-		qt_x_user_time = _event->xbutton.time;
-	    else // KeyPress
-		qt_x_user_time = _event->xkey.time;
-	    QWidget* w = activeWindow();
-	    if( w ) {
-    		XChangeProperty( qt_xdisplay(), w->winId(), kde_net_wm_user_time, XA_CARDINAL,
-        	    32, PropModeReplace, (unsigned char*)&qt_x_user_time, 1 );
-    		timeval tv;
-		gettimeofday( &tv, NULL );
-		unsigned long now = tv.tv_sec * 10 + tv.tv_usec / 100000;
-		XChangeProperty(qt_xdisplay(), w->winId(),
-			    atom_KdeNetUserTime, XA_CARDINAL,
-			    32, PropModeReplace, (unsigned char *)&now, 1);
-	    }
-	}
-	break;
         case ClientMessage:
         {
-#if KDE_IS_VERSION( 3, 2, 91 )
+#if KDE_IS_VERSION( 3, 3, 91 )
 #warning This should be already in Qt, check.
 #endif
         // Workaround for focus stealing prevention not working when dragging e.g. text from KWrite
@@ -1584,7 +1574,7 @@ bool KApplication::x11EventFilter( XEvent *_event )
                     && _event->xclient.data.l[ 4 ] == 0
                     && _event->xclient.data.l[ 3 ] != 0 )
                     {
-                    if( qt_x_user_time == 0 
+                    if( qt_x_user_time == 0
                         || ( _event->xclient.data.l[ 3 ] - qt_x_user_time ) < 100000U )
                         { // and the timestamp looks reasonable
                         qt_x_user_time = _event->xclient.data.l[ 3 ]; // update our qt_x_user_time from it
@@ -1715,7 +1705,20 @@ void KApplication::updateUserTimestamp( unsigned long time )
 
 unsigned long KApplication::userTimestamp() const
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     return qt_x_user_time;
+#else
+    return 0;
+#endif
+}
+
+void KApplication::updateRemoteUserTimestamp( const QCString& dcopId, unsigned long time )
+{
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    if( time == 0 )
+        time = qt_x_user_time;
+    DCOPRef( dcopId, "MainApplication-Interface" ).call( "updateUserTimestamp", time );
+#endif
 }
 
 void KApplication::invokeEditSlot( const char *slot )
@@ -1973,7 +1976,7 @@ void KApplication::propagateSettings(SettingsCategory arg)
     KConfigGroupSaver saver( config, "KDE" );
 
     int num = config->readNumEntry("CursorBlinkRate", QApplication::cursorFlashTime());
-    if (num < 200)
+    if ((num != 0) && (num < 200))
         num = 200;
     if (num > 2000)
         num = 2000;
@@ -2020,6 +2023,8 @@ void KApplication::installKDEPropertyMap()
     kdeMap->insert( "KColorButton", "color" );
     kdeMap->insert( "KComboBox", "currentItem" );
     kdeMap->insert( "KDatePicker", "date" );
+    kdeMap->insert( "KDateWidget", "date" );
+    kdeMap->insert( "KDateTimeWidget", "dateTime" );
     kdeMap->insert( "KEditListBox", "currentItem" );
     kdeMap->insert( "KFontCombo", "family" );
     kdeMap->insert( "KFontRequester", "font" );
@@ -2041,6 +2046,7 @@ void KApplication::installKDEPropertyMap()
     #endif
     //#if QT_VERSION < 0x030300
       // Temp til fixed in QT then enable ifdef with the correct version num
+      kdeMap->insert( "QGroupBox", "checked" );
       kdeMap->insert( "QTabWidget", "currentPage" );
     //#endif
     QSqlPropertyMap::installDefaultMap( kdeMap );
@@ -2074,8 +2080,12 @@ void KApplication::invokeHelp( const QString& anchor,
    {
        if (startServiceByDesktopName("khelpcenter", url, &error, 0, 0, startup_id, false))
        {
-           kdWarning() << "Could not launch help:\n" << error << endl;
-           return;
+           if (Tty != kapp->type())
+               QMessageBox::critical(kapp->mainWidget(), i18n("Could not Launch Help Center"),
+               i18n("Could not launch the KDE Help Center:\n\n%1").arg(error), i18n("&OK"));
+           else
+               kdWarning() << "Could not launch help:\n" << error << endl;
+	   return;
        }
    }
    else
@@ -2104,7 +2114,11 @@ void KApplication::invokeHTMLHelp( const QString& _filename, const QString& topi
    {
        if (startServiceByDesktopName("khelpcenter", url, &error, 0, 0, "", false))
        {
-           kdWarning() << "Could not launch help:\n" << error << endl;
+           if (Tty != kapp->type())
+               QMessageBox::critical(kapp->mainWidget(), i18n("Could not Launch Help Center"),
+               i18n("Could not launch the KDE Help Center:\n\n%1").arg(error), i18n("&OK"));
+           else
+               kdWarning() << "Could not launch help:\n" << error << endl;
            return;
        }
    }
@@ -2131,8 +2145,14 @@ void KApplication::invokeMailer(const KURL &mailtoURL)
 
 void KApplication::invokeMailer(const KURL &mailtoURL, const QCString& startup_id )
 {
-   QString address = KURL::decode_string(mailtoURL.path()), subject, cc, bcc, body, attach;
+    return invokeMailer( mailtoURL, startup_id, false);
+}
+
+void KApplication::invokeMailer(const KURL &mailtoURL, const QCString& startup_id, bool allowAttachments )
+{
+   QString address = KURL::decode_string(mailtoURL.path()), subject, cc, bcc, body;
    QStringList queries = QStringList::split('&', mailtoURL.query().mid(1));
+   QStringList attachURLs;
    for (QStringList::Iterator it = queries.begin(); it != queries.end(); ++it)
    {
      QString q = (*it).lower();
@@ -2140,19 +2160,25 @@ void KApplication::invokeMailer(const KURL &mailtoURL, const QCString& startup_i
        subject = KURL::decode_string((*it).mid(8));
      else
      if (q.startsWith("cc="))
-       cc = KURL::decode_string((*it).mid(3));
+       cc = cc.isEmpty()? KURL::decode_string((*it).mid(3)): cc + ',' + KURL::decode_string((*it).mid(3));
      else
      if (q.startsWith("bcc="))
-       bcc = KURL::decode_string((*it).mid(4));
+       bcc = bcc.isEmpty()? KURL::decode_string((*it).mid(4)): bcc + ',' + KURL::decode_string((*it).mid(4));
      else
      if (q.startsWith("body="))
        body = KURL::decode_string((*it).mid(5));
-     //else
-     //  if (q.startsWith("attach="))
-     //    attach = KURL::decode_string((*it).mid(7));
+     else
+     if (allowAttachments && q.startsWith("attach="))
+       attachURLs.push_back(KURL::decode_string((*it).mid(7)));
+     else
+     if (allowAttachments && q.startsWith("attachment="))
+       attachURLs.push_back(KURL::decode_string((*it).mid(11)));
+     else
+     if (q.startsWith("to="))
+       address = address.isEmpty()? KURL::decode_string((*it).mid(3)): address + ',' + KURL::decode_string((*it).mid(3));
    }
 
-   invokeMailer( address, cc, bcc, subject, body, QString::null, QStringList(), startup_id );
+   invokeMailer( address, cc, bcc, subject, body, QString::null, attachURLs, startup_id );
 }
 
 void KApplication::invokeMailer(const QString &to, const QString &cc, const QString &bcc,
@@ -2179,13 +2205,12 @@ void KApplication::invokeMailer(const QString &to, const QString &cc, const QStr
        || command.endsWith("/kmail"))
      command = QString::fromLatin1("kmail --composer -s %s -c %c -b %b --body %B --attach %A -- %t");
 
-   // TODO: Take care of the preferred terminal app (instead of hardcoding
-   // Konsole), this will probably require a rewrite of the configurable
-   // terminal client option because the placeholder for the program which
-   // has to be executed by the terminal has to be supplied (e.g. something
-   // like '/opt/kde2/bin/konsole -e %p'). - Frerich
    if (config.readBoolEntry("TerminalClient", false))
-      command = "konsole -e " + command;
+   {
+     KConfigGroup confGroup( KGlobal::config(), "General" );
+     QString preferredTerminal = confGroup.readPathEntry("TerminalApplication", "konsole");
+     command = preferredTerminal + " -e " + command;
+   }
 
    QStringList cmdTokens = KShell::splitArgs(command);
    QString cmd = cmdTokens[0];
@@ -2231,7 +2256,11 @@ void KApplication::invokeMailer(const QString &to, const QString &cc, const QStr
    // TODO this should check if cmd has a .desktop file, and use data from it, together
    // with sending more ASN data
    if (kdeinitExec(cmd, cmdTokens, &error, NULL, startup_id ))
-      kdWarning() << "Could not launch mail client:\n" << error << endl;
+     if (Tty != kapp->type())
+       QMessageBox::critical(kapp->mainWidget(), i18n("Could not Launch Mail Client"),
+             i18n("Could not launch the mail client:\n\n%1").arg(error), i18n("&OK"));
+     else
+       kdWarning() << "Could not launch mail client:\n" << error << endl;
 }
 
 
@@ -2244,9 +2273,13 @@ void KApplication::invokeBrowser( const QString &url, const QCString& startup_id
 {
    QString error;
 
-   if (startServiceByDesktopName("kfmclient", url, &error, 0, 0, startup_id, true))
+   if (startServiceByDesktopName("kfmclient", url, &error, 0, 0, startup_id, false))
    {
-      kdWarning() << "Could not launch browser:\n" << error << endl;
+      if (Tty != kapp->type())
+          QMessageBox::critical(kapp->mainWidget(), i18n("Could not Launch Browser"),
+               i18n("Could not launch the browser:\n\n%1").arg(error), i18n("&OK"));
+      else
+          kdWarning() << "Could not launch browser:\n" << error << endl;
       return;
    }
 }
@@ -2327,7 +2360,9 @@ startServiceInternal( const QCString &function,
        envs.append( QCString("DISPLAY=") + dpystring );
    }
 #endif
-   stream << envs << startup_id;
+   stream << envs;
+   // make sure there is id, so that user timestamp exists
+   stream << ( startup_id.isEmpty() ? KStartupInfo::createNewStartupId() : startup_id );
    if( function.left( 12 ) != "kdeinit_exec" )
        stream << noWait;
 
@@ -2553,39 +2588,12 @@ bool checkAccess(const QString& pathname, int mode)
 
 void KApplication::setTopWidget( QWidget *topWidget )
 {
-  if( topWidget != 0 )
-  {
-#ifdef Q_WS_X11 // FIXME(E): Implement for Qt/Embedded
-    Window leader = topWidget->winId();
-    char* argv = const_cast< char* >( KCmdLineArgs::appName());
-    XSetCommand(display, leader, &argv, 1);
-    // this hints thing may go after Qt always sets window_group
-    XWMHints *hints = XGetWMHints(display, topWidget->winId());
-    if (hints)
-    {
-        if (!(hints->flags & WindowGroupHint))
-        {
-            hints->window_group = leader;
-            hints->flags |= WindowGroupHint;
-        }
-        if (!(hints->flags & InputHint))
-        {
-            hints->input = True;
-            hints->flags |= InputHint;
-        }
-        XSetWMHints(display, topWidget->winId(), hints);
-        XFree(reinterpret_cast<char *>(hints));
-    }
+  if( !topWidget )
+      return;
 
-#endif
     // set the specified caption
     if ( !topWidget->inherits("KMainWindow") ) { // KMainWindow does this already for us
         topWidget->setCaption( caption() );
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
-//#ifndef Q_WS_QWS // FIXME(E): Implement for Qt/Embedded
-        NETWinInfo info(qt_xdisplay(), topWidget->winId(), qt_xrootwin(), NET::WMName );
-        info.setName( caption().utf8().data() );
-#endif
     }
 
     // set the specified icons
@@ -2597,7 +2605,6 @@ void KApplication::setTopWidget( QWidget *topWidget )
     // set the app startup notification window property
     KStartupInfo::setWindowStartupId( topWidget->winId(), startupId());
 #endif
-  }
 }
 
 QCString KApplication::startupId() const
@@ -2610,7 +2617,14 @@ void KApplication::setStartupId( const QCString& startup_id )
     if( startup_id.isEmpty())
         d->startup_id = "0";
     else
+        {
         d->startup_id = startup_id;
+        KStartupInfoId id;
+        id.initId( startup_id );
+        long timestamp = id.timestamp();
+        if( timestamp != 0 )
+            updateUserTimestamp( timestamp );
+        }
 }
 
 // read the startup notification env variable, save it and unset it in order
@@ -2684,7 +2698,7 @@ bool KApplication::authorizeKAction(const char *action)
 
 bool KApplication::authorizeControlModule(const QString &menuId)
 {
-   if (menuId.isEmpty())
+   if (menuId.isEmpty() || kde_kiosk_exception)
       return true;
    KConfig *config = KGlobal::config();
    KConfigGroupSaver saver( config, "KDE Control Module Restrictions" );
@@ -2786,7 +2800,7 @@ void KApplication::allowURLAction(const QString &action, const KURL &_baseURL, c
 {
   if (authorizeURLAction(action, _baseURL, _destURL))
      return;
-     
+
   d->urlActionRestrictions.append(new KApplicationPrivate::URLActionRule
         ( action, _baseURL.protocol(), _baseURL.host(), _baseURL.path(-1),
                   _destURL.protocol(), _destURL.host(), _destURL.path(-1), true));
