@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include <kurl.h>
 #include <stdio.h>
 #include <kapplication.h>
@@ -6,8 +8,10 @@
 #include <kglobal.h>
 #include <kcharsets.h>
 #include <qtextcodec.h>
+#include <qdatastream.h>
+#include <assert.h>
 
-bool check(QString txt, QString a, QString b)
+static bool check(QString txt, QString a, QString b)
 {
   if (a.isEmpty())
      a = QString::null;
@@ -29,6 +33,7 @@ extern void qt_set_locale_codec( QTextCodec *codec );
 
 int main(int argc, char *argv[])
 {
+  KApplication::disableAutoDcopRegistration();
   KApplication app(argc,argv,"kurltest",false,false);
 
   KURL::List lst;
@@ -37,6 +42,7 @@ int main(int argc, char *argv[])
   check( "KURL::isMalformed()", emptyURL.isMalformed() ? "TRUE":"FALSE", "TRUE");
   check( "KURL::isValid()", emptyURL.isValid() ? "TRUE":"FALSE", "FALSE");
   check( "KURL::isEmpty()", emptyURL.isEmpty() ? "TRUE":"FALSE", "TRUE");
+  check( "prettyURL()", emptyURL.prettyURL(), "");
 
   emptyURL = "";
   check( "KURL::isMalformed()", emptyURL.isMalformed() ? "TRUE":"FALSE", "TRUE");
@@ -197,7 +203,7 @@ int main(int argc, char *argv[])
   check("KURL::directory(false,false)", udir.directory(false,false), "/home/dfaure/");
   check("KURL::directory(true,false)", udir.directory(true,false), "/home/dfaure");
 
-  KURL u2("/home/dfaure/");
+  KURL u2( QCString("/home/dfaure/") );
   printf("\n* URL is %s\n",u2.url().ascii());
   // not ignoring trailing slash
   check("KURL::directory(false,false)", u2.directory(false,false), "/home/dfaure/");
@@ -232,6 +238,17 @@ int main(int argc, char *argv[])
       KURL newUrl = tmpurl;
       newUrl.setFileName( "mynewdir" );
   check("KURL::setFileName() special", newUrl.url(), "file:/home/dfaure/mynewdir");
+  // addPath tests
+  newUrl.addPath( "subdir" );
+  check("KURL::addPath(\"subdir\")", newUrl.url(), "file:/home/dfaure/mynewdir/subdir");
+  newUrl.addPath( "/foo/" );
+  check("KURL::addPath(\"/foo/\")", newUrl.url(), "file:/home/dfaure/mynewdir/subdir/foo/");
+  u2 = "http://www.kde.org"; // no path
+  u2.addPath( "subdir" );
+  check("KURL::addPath(\"subdir\")", u2.url(), "http://www.kde.org/subdir");
+  u2.addPath( "" );
+  check("KURL::addPath(\"subdir\")", u2.url(), "http://www.kde.org/subdir"); // unchanged
+
   // even more tricky
   u2 = "print:/specials/Print%20To%20File%20(PDF%2FAcrobat)";
   printf("\n* URL is %s\n",u2.url().ascii());
@@ -268,6 +285,12 @@ int main(int argc, char *argv[])
   check("urlcmp(only hash difference)", urlcmp(ucmp2,ucmp3)?"ko":"ok","ok");
   check("urlcmp(only hash difference, ignore_ref)", urlcmp(ucmp2,ucmp3,false,true)?"ok":"ko","ok");
   check("urlcmp(slash and hash difference, ignore_trailing, ignore_ref)", urlcmp(ucmp2,ucmp3,true,true)?"ok":"ko","ok");
+  check("urlcmp(empty, empty)", urlcmp("","",false,true)?"ok":"ko","ok");
+  check("urlcmp(empty, empty)", urlcmp("","")?"ok":"ko","ok");
+  check("urlcmp(empty, not empty)", urlcmp("",ucmp1)?"ok":"ko","ko");
+  check("urlcmp(empty, not empty)", urlcmp("",ucmp1,false,true)?"ok":"ko","ko");
+  check("urlcmp(malformed, not empty)", urlcmp("file",ucmp1)?"ok":"ko","ko");
+  check("urlcmp(malformed, not empty)", urlcmp("file",ucmp1,false,true)?"ok":"ko","ko");
 
   KURL ftpUrl ( "ftp://ftp.de.kde.org" );
   printf("\n* URL is %s\n",ftpUrl.url().latin1());
@@ -331,6 +354,22 @@ int main(int argc, char *argv[])
      KURL waba2( waba1, "#");
      check("http: Relative URL, with empty reference", waba2.url(), "http://www.website.com/directory/?hello#");
   }
+  {
+     KURL waba2( waba1, "");
+     check("http: Empty relative URL", waba2.url(), "http://www.website.com/directory/?hello#ref");
+  }
+  {
+     KURL base( "http://faure@www.kde.org" ); // no path
+     KURL waba2( base, "filename.html");
+     check("http: Relative URL, orig URL had no path", waba2.url(), "http://faure@www.kde.org/filename.html");
+  }
+  {
+     KURL base( "http://faure:pass@www.kde.org:81?query" );
+     KURL rel1( base, "http://www.kde.org/bleh/"); // same host
+     check("http: Relative URL, orig URL had username", rel1.url(), "http://faure:pass@www.kde.org/bleh/");
+     KURL rel2( base, "http://www.yahoo.org"); // different host
+     check("http: Relative URL, orig URL had username", rel2.url(), "http://www.yahoo.org");
+  }
 
   waba1 = "http://www.website.com/directory/filename?bla#blub";
   {
@@ -361,10 +400,23 @@ int main(int argc, char *argv[])
      KURL waba2( waba1, "relative.html#with_reference");
      check("http: Relative URL, with reference", waba2.url(), "http://www.website.com/directory/relative.html#with_reference");
   }
+  {
+      KURL waba2( waba1, "http:/relative.html"); // "rfc 1606 loophole"
+      check("http: Strange relative URL", waba2.url(), "http://www.website.com/relative.html");
+  }
   waba1.setUser("waldo");
   check("http: Set user", waba1.url(), "http://waldo@www.website.com/directory/filename?bla#blub");
   waba1.setUser("waldo/bastian");
   check("http: Set user with slash in it", waba1.url(), "http://waldo%2Fbastian@www.website.com/directory/filename?bla#blub");
+  waba1.setRef( QString::null );
+  waba1.setPass( "pass" );
+  waba1.setDirectory( "/foo" );
+  waba1.setProtocol( "https" );
+  waba1.setHost( "web.com" );
+  waba1.setPort( 881 );
+  check("http: setRef/setPass/setDirectory/setHost/setPort", waba1.url(), "https://waldo%2Fbastian:pass@web.com:881/foo/?bla");
+  waba1.setDirectory( "/foo/" );
+  check("http: setDirectory #2", waba1.url(), "https://waldo%2Fbastian:pass@web.com:881/foo/?bla");
 
   // Empty queries should be preserved!
   waba1 = "http://www.kde.org/cgi/test.cgi?";
@@ -388,6 +440,8 @@ int main(int argc, char *argv[])
   waba1.setQuery("hello:My Value");
   check("http: URL with ':' in query", waba1.url(),
         "http://www.kde.org/cgi/test.cgi?hello:My%20Value");
+  check("upURL() removes query", waba1.upURL().url(),
+        "http://www.kde.org/cgi/test.cgi");
 
   // URLs who forgot to encode spaces in the query.
   waba1 = "http://www.kde.org/cgi/test.cgi?hello=My Value+20";
@@ -478,6 +532,28 @@ int main(int argc, char *argv[])
   check("http: IPV6 without path; port", QString::number( waba1.port() ), "81" );
   check("http: IPV6 without path; ref", waba1.ref(), "ref");
 
+  // Streaming operators
+  KURL origURL( "http://www.website.com/directory/?#ref" );
+  waba1 = "http://[::ffff:129.144.52.38]:81?query";
+  QByteArray buffer;
+  {
+      QDataStream stream( buffer, IO_WriteOnly );
+      stream << origURL
+             << KURL( "file:" ) // an invalid one
+             << waba1; // the IPv6 one
+  }
+  {
+      QDataStream stream( buffer, IO_ReadOnly );
+      KURL restoredURL;
+      stream >> restoredURL;
+      check( "Streaming valid URL", origURL.url(), restoredURL.url() );
+      stream >> restoredURL;
+      check( "Streaming invalid URL", restoredURL.isValid()?"valid":"malformed", "malformed" );
+      check( "Streaming invalid URL", restoredURL.url(), "file:" );
+      stream >> restoredURL;
+      check( "Streaming ipv6 URL with query", restoredURL.url(), waba1.url() );
+  }
+
   // Broken stuff
   waba1 = "file:a";
   check("Broken stuff #1 path", waba1.path(), "a");
@@ -514,6 +590,13 @@ int main(int argc, char *argv[])
   check("Broken stuff #5 empty", broken.isEmpty()?"EMPTY":"NOT", "NOT");
   check("Broken stuff #5 valid", broken.isValid()?"VALID":"MALFORMED", "MALFORMED");
   check("Broken stuff #5 path", broken.path(), "");
+  broken = "file";
+  check("Broken stuff #6 valid", broken.isValid()?"VALID":"MALFORMED", "MALFORMED");
+
+  broken = "LABEL=USB_STICK"; // 71430, can we use KURL for this?
+  check("Broken stuff #6 valid", broken.isValid()?"VALID":"MALFORMED", "MALFORMED");
+  check("Broken stuff #6 empty", broken.isEmpty()?"EMPTY":"NOT", "NOT");
+  check("Broken stuff #6 path", broken.path(), "");
 
 #if 0 // BROKEN?
   // UNC like names
@@ -547,6 +630,8 @@ int main(int argc, char *argv[])
   check("mailto: URL, general form", umail2.protocol(), "mailto");
   check("mailto: URL, general form", umail2.path(), "Faure David <faure@kde.org>");
   check("isRelativeURL(\"mailto:faure@kde.org\")", KURL::isRelativeURL("mailto:faure@kde.org") ? "yes" : "no", "no");
+  KURL umail3 ( "mailto:" );
+  check("mailto: invalid URL", umail3.isValid()?"valid":"malformed", "malformed");
 
   check("man: URL, is relative", KURL::isRelativeURL("man:mmap") ? "true" : "false", "false");
   check("javascript: URL, is relative", KURL::isRelativeURL("javascript:doSomething()") ? "true" : "false", "false");
@@ -575,6 +660,18 @@ int main(int argc, char *argv[])
   uloc = KURL("file:/home/dfaure/konqtests/Mat%C3%A9riel", 106);
   check("UTF8 path", uloc.path(), "/home/dfaure/konqtests/Matriel");
 
+  // fromPathOrURL tests
+  uloc = KURL::fromPathOrURL( "/home/dfaure/konqtests/Mat%E9riel" );
+  check("fromPathOrURL path", uloc.path(), "/home/dfaure/konqtests/Mat%E9riel");
+  uloc = KURL::fromPathOrURL( "http://www.kde.org" );
+  check("fromPathOrURL url", uloc.url(), "http://www.kde.org");
+  uloc = KURL::fromPathOrURL( "www.kde.org" );
+  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+  uloc = KURL::fromPathOrURL( "index.html" );
+  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+  uloc = KURL::fromPathOrURL( "" );
+  check("fromPathOrURL url", uloc.isValid()?"valid":"malformed", "malformed");
+
 #if QT_VERSION < 300
   qt_set_locale_codec( KGlobal::charsets()->codecForName( "koi8-r" ) );
 #else
@@ -637,17 +734,30 @@ int main(int argc, char *argv[])
   // Needed for #49616
   check( "encode_string('C++')", KURL::encode_string( "C++" ), "C%2B%2B" );
   check( "decode_string('C%2B%2B')", KURL::decode_string( "C%2B%2B" ), "C++" );
+  check( "decode_string('C%00A')", KURL::decode_string( "C%00%A" ), "C" ); // we stop at %00
 
   check( "encode_string('%')", KURL::encode_string( "%" ), "%25" );
   check( "encode_string(':')", KURL::encode_string( ":" ), "%3A" );
 
   KURL amantia( "http://%E1.foo" );
   check("amantia.isValid()", amantia.isValid() ? "true" : "false", "true");
-  check("amantia.url()", amantia.url(), "http://"); // OK, an escaped char in a hostname is really evil, not sure what should happen.
+#ifdef HAVE_IDNA_H
+  check("amantia.url()", amantia.url(), "http://xn--80a.foo");   // Non-ascii is allowed in IDN domain names.
+#else
+  check("amantia.url()", amantia.url(), "http://?.foo"); // why not
+#endif
 
   KURL smb("smb://domain;username:password@server/share");
   check("smb.isValid()", smb.isValid() ? "true" : "false", "true");
   check("smb.user()", smb.user(), "domain;username");
+  smb = "smb:/";
+  check("smb:/", smb.isValid()?"VALID":"MALFORMED", "VALID");
+  smb = "smb://"; // kurl.cpp rev 1.106
+  check("smb://", smb.isValid()?"VALID":"MALFORMED", "MALFORMED");
+  smb = "smb://host";
+  check("smb://host", smb.isValid()?"VALID":"MALFORMED", "VALID");
+  smb = "smb:///";
+  check("smb:///", smb.isValid()?"VALID":"MALFORMED", "VALID");
 
   KURL weird;
   weird = "http://strange<hostname>/";
@@ -697,7 +807,7 @@ int main(int argc, char *argv[])
   check("utf8_2.fileName()", utf8_2.fileName(), QString::fromLatin1("15/Geantra.wav"));
 
   KURL url_newline_1("http://www.foo.bar/foo/bar\ngnork");
-  check("url_newline_1.url()", url_newline_1.url(), QString::fromLatin1("http://www.foo.bar/foo/bar%0Agnork")); 
+  check("url_newline_1.url()", url_newline_1.url(), QString::fromLatin1("http://www.foo.bar/foo/bar%0Agnork"));
 
   KURL url_newline_2("http://www.foo.bar/foo?bar\ngnork");
   check("url_newline_2.url()", url_newline_2.url(), QString::fromLatin1("http://www.foo.bar/foo?bar%0Agnork"));
@@ -722,7 +832,7 @@ int main(int argc, char *argv[])
   check("local_file_5.url()", local_file_5.url(), "file:/foo%3Fbar");
 
   QString basePath = "/home/bastian";
-  
+
   check("relativePath(\"/home/bastian\", \"/home/bastian\")", KURL::relativePath(basePath, "/home/bastian"), "./");
   bool b;
   check("relativePath(\"/home/bastian\", \"/home/bastian/src/plugins\")", KURL::relativePath(basePath, "/home/bastian/src/plugins", &b), "./src/plugins");
@@ -734,12 +844,14 @@ int main(int argc, char *argv[])
 
   check("relativePath(\"/\", \"/\")", KURL::relativePath("/", "/"), "./");
   check("relativePath(\"/\", \"/home/bastian\")", KURL::relativePath("/", "/home/bastian"), "./home/bastian");
+  check("relativePath(\"\", \"/home/bastian\")", KURL::relativePath("", "/home/bastian"), "/home/bastian");
 
   baseURL = "http://www.kde.org/index.html";
   check("relativeURL(\"http://www.kde.org/index.html\", \"http://www.kde.org/index.html#help\")", KURL::relativeURL(baseURL, "http://www.kde.org/index.html#help"), "#help");
   check("relativeURL(\"http://www.kde.org/index.html\", \"http://www.kde.org/index.html?help=true\")", KURL::relativeURL(baseURL, "http://www.kde.org/index.html?help=true"), "index.html?help=true");
   check("relativeURL(\"http://www.kde.org/index.html\", \"http://www.kde.org/contact.html\")", KURL::relativeURL(baseURL, "http://www.kde.org/contact.html"), "contact.html");
   check("relativeURL(\"http://www.kde.org/index.html\", \"ftp://ftp.kde.org/pub/kde\")", KURL::relativeURL(baseURL, "ftp://ftp.kde.org/pub/kde"), "ftp://ftp.kde.org/pub/kde");
+  check("relativeURL(\"http://www.kde.org/index.html\", \"http://www.kde.org/index.html\")", KURL::relativeURL(baseURL, "http://www.kde.org/index.html"), "./");
 
   baseURL = "http://www.kde.org/info/index.html";
   check("relativeURL(\"http://www.kde.org/info/index.html\", \"http://www.kde.org/bugs/contact.html\")", KURL::relativeURL(baseURL, "http://www.kde.org/bugs/contact.html"), "../bugs/contact.html");
@@ -762,7 +874,12 @@ int main(int argc, char *argv[])
   weird = "ftp://user%40host.com@ftp.host.com/var/www/";
   check("user()?", weird.user(), "user@host.com" );
   check("host()?", weird.host(), "ftp.host.com" );
-  check("KURL::upURL()", weird.upURL().url(), "ftp://user%40host.com@ftp.host.com/var/");
+  KURL up = weird.upURL();
+  check("KURL::upURL()", up.url(), "ftp://user%40host.com@ftp.host.com/var/");
+  up = up.upURL();
+  check("KURL::upURL()", up.url(), "ftp://user%40host.com@ftp.host.com/");
+  up = up.upURL();
+  check("KURL::upURL()", up.url(), "ftp://user%40host.com@ftp.host.com/"); // unchanged
 
   KURL ldap = "ldap://host.com:6666/o=University%20of%20Michigan,c=US??sub?(cn=Babs%20Jensen)";
   check("host()?", ldap.host(), "host.com");
@@ -774,11 +891,19 @@ int main(int argc, char *argv[])
   check("query()?", ldap.query(), "??sub?(cn=Karl%20Marx)");
   check("url()?", ldap.url(), "ldap://host.com:6666/o=University%20of%20Michigan,c=US??sub?(cn=Karl%20Marx)");
 
+  KURL leo = "data:text/html,http://www.invalid/";
+  check("data URL: isValid", leo.isValid()?"valid":"malformed", "valid" );
+  check("data URL: protocol", leo.protocol(), "data" );
+  check("data URL: url", leo.url(), "data:text/html,http://www.invalid/" );
+  check("data URL: path", leo.path(), "text/html,http://www.invalid/" );
+
   // URI Mode tests
   url1 = "http://www.foobar.com/";
   check("KURL(\"http://www.foobar.com/\").uriMode()", QString::number(url1.uriMode()), QString::number(KURL::URL));
   url1 = "mailto:user@host.com";
   check("KURL(\"mailto:user@host.com\").uriMode()", QString::number(url1.uriMode()), QString::number(KURL::Mailto));
+  check("KURL(\"mailto:user@host.com\").url()", url1.url(), "mailto:user@host.com");
+  check("KURL(\"mailto:user@host.com\").url(0, 106)", url1.url(0, 106), "mailto:user@host.com");
   url1 = "data:text/plain,foobar?gazonk=flarp";
   check("KURL(\"data:text/plain,foobar?gazonk=flarp\").uriMode()", QString::number(url1.uriMode()), QString::number(KURL::RawURI));
   check("KURL(\"data:text/plain,foobar?gazonk=flarp\").path()", url1.path(), "text/plain,foobar?gazonk=flarp");
