@@ -19,9 +19,9 @@
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
@@ -32,6 +32,7 @@
 
 #include <qlineedit.h>
 #include <kcompletion.h>
+#include <kdemacros.h>
 
 class QPopupMenu;
 
@@ -41,18 +42,18 @@ class KURL;
 /**
  * An enhanced QLineEdit widget for inputting text.
  *
- * @sect Detail
+ * \b Detail \n
  *
- * This widget inherits from @ref QLineEdit and implements the following
- * additional functionalities: q completion object that provides both
+ * This widget inherits from QLineEdit and implements the following
+ * additional functionalities: a completion object that provides both
  * automatic and manual text completion as well as multiple match iteration
  * features, configurable key-bindings to activate these features and a
  * popup-menu item that can be used to allow the user to set text completion
  * modes on the fly based on their preference.
  *
- * To support these new features @ref KLineEdit also emits a few more
- * additional signals.  These are: @ref completion( const QString& ),
- * textRotation( KeyBindingType ), and @ref returnPressed( const QString& ).
+ * To support these new features KLineEdit also emits a few more
+ * additional signals.  These are: completion( const QString& ),
+ * textRotation( KeyBindingType ), and returnPressed( const QString& ).
  * The completion signal can be connected to a slot that will assist the
  * user in filling out the remaining text.  The text rotation signal is
  * intended to be used to iterate through the list of all possible matches
@@ -62,87 +63,100 @@ class KURL;
  * appropriate.
  *
  * This widget by default creates a completion object when you invoke
- * the @ref completionObject( bool ) member function for the first time or
- * use @ref setCompletionObject( KCompletion*, bool ) to assign your own
+ * the completionObject( bool ) member function for the first time or
+ * use setCompletionObject( KCompletion*, bool ) to assign your own
  * completion object.  Additionally, to make this widget more functional,
- * @ref KLineEdit will by default handle the text rotation and completion
+ * KLineEdit will by default handle the text rotation and completion
  * events internally when a completion object is created through either one
  * of the methods mentioned above.  If you do not need this functionality,
- * simply use @ref KCompletionBase::setHandleSignals( bool ) or set the
- * boolean parameter in the above functions to FALSE.
+ * simply use KCompletionBase::setHandleSignals( bool ) or set the
+ * boolean parameter in the above functions to false.
  *
  * The default key-bindings for completion and rotation is determined
- * from the global settings in @ref KStdAccel.  These values, however,
- * can be overriden locally by invoking @ref KCompletionBase::setKeyBinding().
+ * from the global settings in KStdAccel.  These values, however,
+ * can be overridden locally by invoking KCompletionBase::setKeyBinding().
  * The values can easily be reverted back to the default setting, by simply
- * calling @ref useGlobalSettings(). An alternate method would be to default
- * individual key-bindings by usning @ref setKeyBinding() with the default
+ * calling useGlobalSettings(). An alternate method would be to default
+ * individual key-bindings by using setKeyBinding() with the default
  * second argument.
  *
- * NOTE that if the @p EchoMode for this widget is set to something other
- * than @p QLineEdit::Normal, the completion mode will always be defaulted
- * to @ref KGlobal::CompletionNone.  This is done purposefully to guard
- * against protected entries such as passwords being cached in @ref KCompletion's
- * list. Hence, if the @p EchoMode is not @ref QLineEdit::Normal, the completion
- * mode is automatically disabled.
+ * If @p EchoMode for this widget is set to something other than @p QLineEdit::Normal, 
+ * the completion mode will always be defaulted to KGlobalSettings::CompletionNone.  
+ * This is done purposefully to guard against protected entries such as passwords being
+ * cached in KCompletion's list. Hence, if the @p EchoMode is not QLineEdit::Normal, the
+ * completion mode is automatically disabled.
  *
- * @sect Useage
+ * A read-only KLineEdit will have the same background color as a 
+ * disabled KLineEdit, but its foreground color will be the one used 
+ * for the read-write mode. This differs from QLineEdit's implementation
+ * and is done to give visual distinction between the three different modes: 
+ * disabled, read-only, and read-write.
+ *
+ * \b Usage \n
  *
  * To enable the basic completion feature :
  *
- * <pre>
+ * \code
  * KLineEdit *edit = new KLineEdit( this, "mywidget" );
  * KCompletion *comp = edit->completionObject();
  * // Connect to the return pressed signal - optional
  * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
- * </pre>
+ * \endcode
  *
  * To use a customized completion objects or your
  * own completion object :
  *
- * <pre>
+ * \code
  * KLineEdit *edit = new KLineEdit( this,"mywidget" );
  * KURLCompletion *comp = new KURLCompletion();
  * edit->setCompletionObject( comp );
  * // Connect to the return pressed signal - optional
  * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&));
- * </pre>
+ * \endcode
  *
- * Note that you have to either delete the allocated completion object
- * when you don't need it anymore, or call
- * setAutoDeleteCompletionObject( true );
+ * Note if you specify your own completion object you have to either delete 
+ * it when you don't need it anymore, or you can tell KLineEdit to delete it 
+ * for you:
+ * \code 
+ * edit->setAutoDeleteCompletionObject( true );
+ * \endcode
  *
- * @sect Miscellaneous function calls :
+ * <b>Miscellaneous function calls :</b>\n
  *
- * <pre>
- * // Tell the widget not to handle completion and
- * // iteration internally.
+ * \code
+ * // Tell the widget to not handle completion and iteration automatically.
  * edit->setHandleSignals( false );
- * // Set your own completion key for manual completions.
+ *
+ * // Set your own key-bindings for a text completion mode.
  * edit->setKeyBinding( KCompletionBase::TextCompletion, Qt::End );
+ *
  * // Hide the context (popup) menu
  * edit->setContextMenuEnabled( false );
- * // Temporarly disable signal emitions
- * // (both completion & iteration signals)
- * edit->disableSignals();
- * // Default the key-bindings to system settings.
- * edit->useGlobalKeyBindings();
- * </pre>
  *
- * @short An enhanced single line input widget.
+ * // Temporarily disable signal (both completion & iteration) emitions
+ * edit->disableSignals();
+ *
+ * // Default the key-bindings back to the default system settings.
+ * edit->useGlobalKeyBindings();
+ * \endcode
+ *
  * @author Dawit Alemayehu <adawit@kde.org>
  */
 
 class KLineEdit : public QLineEdit, public KCompletionBase
 {
-  Q_OBJECT
-  Q_PROPERTY( bool contextMenuEnabled READ isContextMenuEnabled WRITE setContextMenuEnabled )
-  Q_PROPERTY( bool urlDropsEnabled READ isURLDropsEnabled WRITE setURLDropsEnabled )
+    friend class KComboBox;
+
+    Q_OBJECT
+    Q_PROPERTY( bool contextMenuEnabled READ isContextMenuEnabled WRITE setContextMenuEnabled )
+    Q_PROPERTY( bool urlDropsEnabled READ isURLDropsEnabled WRITE setURLDropsEnabled )
+    Q_PROPERTY( bool trapEnterKeyEvent READ trapReturnKey WRITE setTrapReturnKey )
+    Q_PROPERTY( bool enableSqueezedText READ isSqueezedTextEnabled WRITE setEnableSqueezedText )
 
 public:
 
     /**
-     * Construct a @ref KLineEdit object with a default text, a parent,
+     * Constructs a KLineEdit object with a default text, a parent,
      * and a name.
      *
      * @param string Text to be shown in the edit widget.
@@ -152,9 +166,8 @@ public:
     KLineEdit( const QString &string, QWidget *parent, const char *name = 0 );
 
     /**
-     * Construct a @ref KLineEdit object with a parent and a name.
+     * Constructs a KLineEdit object with a parent and a name.
      *
-     * @param string Text to be shown in the edit widget.
      * @param parent The parent object of this widget.
      * @param name The name of this widget.
      */
@@ -166,29 +179,29 @@ public:
     virtual ~KLineEdit ();
 
     /**
-     * Sets @p url into the lineedit. It uses @ref KURL::prettyURL() so
+     * Sets @p url into the lineedit. It uses KURL::prettyURL() so
      * that the url is properly decoded for displaying.
      */
     void setURL( const KURL& url );
 
     /**
-     * Put cursor at the end of the string.
+     * Puts the text cursor at the end of the string.
      *
-     * This method is deprecated.  Use @ref QLineEdit::end()
+     * This method is deprecated.  Use QLineEdit::end()
      * instead.
      *
      * @deprecated
-     * @ref QLineEdit::end()
+     * QLineEdit::end()
      */
     void cursorAtEnd() { end( false ); }
 
     /**
-     * Re-implemented from @ref KCompletionBase for internal reasons.
+     * Re-implemented from KCompletionBase for internal reasons.
      *
      * This function is re-implemented in order to make sure that
      * the EchoMode is acceptable before we set the completion mode.
      *
-     * See @ref KCompletionBase::setCompletionMode
+     * See KCompletionBase::setCompletionMode
      */
     virtual void setCompletionMode( KGlobalSettings::Completion mode );
 
@@ -198,11 +211,11 @@ public:
     * Note that when this function is invoked with its argument
     * set to @p true, then both the context menu and the completion
     * menu item are enabled.  If you do not want to the completion
-    * item to be visible simply invoke @ref hideModechanger() right
+    * item to be visible simply invoke hideModechanger() right
     * after calling this method.  Also by default, the context
     * menu is automatically created if this widget is editable. Thus
     * you need to call this function with the argument set to false
-    * if you do not want this behaviour.
+    * if you do not want this behavior.
     *
     * @param showMenu If @p true, show the context menu.
     */
@@ -210,15 +223,13 @@ public:
 
     /**
      * Returns @p true when the context menu is enabled.
-     *
-     * @return @p true If context menu is enabled.
      */
     bool isContextMenuEnabled() const { return m_bEnableMenu; }
 
     /**
      * Enables/Disables handling of URL drops. If enabled and the user
      * drops an URL, the decoded URL will be inserted. Otherwise the default
-     * behaviour of QLineEdit is used, which inserts the encoded URL.
+     * behavior of QLineEdit is used, which inserts the encoded URL.
      *
      * @param enable If @p true, insert decoded URLs
      */
@@ -226,21 +237,17 @@ public:
 
     /**
      * Returns @p true when decoded URL drops are enabled
-     *
-     * @return @p true if decoded URL drops are enabled
      */
     bool isURLDropsEnabled() const;
 
     /**
-     * By default, @ref KComboBox recognizes @p Key_Return and @p Key_Enter and emits
-     * the @ref returnPressed() signals, but it also lets the event pass,
+     * By default, KLineEdit recognizes @p Key_Return and @p Key_Enter and emits
+     * the returnPressed() signals, but it also lets the event pass,
      * for example causing a dialog's default-button to be called.
      *
-     * Call this method with @p trap = @p true to make @p KComboBox stop these
+     * Call this method with @p trap = @p true to make @p KLineEdit stop these
      * events. The signals will still be emitted of course.
      *
-     * Only affects read-writable comboboxes.
-     *
      * @see trapReturnKey()
      */
     void setTrapReturnKey( bool trap );
@@ -256,36 +263,72 @@ public:
     /**
      * Re-implemented for internal reasons.  API not affected.
      *
-     * @reimplemented
      */
     virtual bool eventFilter( QObject *, QEvent * );
 
     /**
      * @returns the completion-box, that is used in completion mode
-     * @ref KGlobalSettings::CompletionPopup.
-     * This method will create a completion-box by calling
-     * @ref makeCompletionBox, if none is there, yet.
+     * KGlobalSettings::CompletionPopup.
+     * This method will create a completion-box if none is there, yet.
+     *
+     * @param create Set this to false if you don't want the box to be created
+     *               i.e. to test if it is available.
      */
-    KCompletionBox * completionBox();
-
-    /**
-     * @returns the completion box or 0L if none is available
-     */
-    KCompletionBox * completionBox( bool create );
+    KCompletionBox * completionBox( bool create = true );
 
     /**
      * Reimplemented for internal reasons, the API is not affected.
      */
     virtual void setCompletionObject( KCompletion *, bool hsig = true );
 
+    /**
+     * Reimplemented for internal reasons, the API is not affected.
+     */
+    virtual void copy() const;
+
+    /**
+     * Enable text squeezing whenever the supplied text is too long.
+     * Only works for "read-only" mode.
+     *
+     * Note that once text squeezing is enabled, @ref QLineEdit::text
+     * and @ref QLineEdit::displayText return the squeezed text. If
+     * you want the original text, use @ref originalText.
+     *
+     * @since 3.2
+     */    
+    void setEnableSqueezedText( bool enable );
+    
+    /**
+     * Returns true if text squeezing is enabled.
+     * This is only valid when the widget is in read-only mode.
+     *
+     * @since 3.2
+     */
+    bool isSqueezedTextEnabled() const;
+    
+    /**
+     * Returns the original text if text squeezing is enabled 
+     * If the widget is not in "read-only" mode, this function
+     * returns the same thing as @ref QLineEdit::text().
+     * 
+     * @since 3.2
+     */
+    QString originalText() const;
+
 signals:
 
+    /**
+     * Emitted whenever the completion box is activated.
+     * @since 3.1
+     */
+    void completionBoxActivated (const QString &);
+
     /**
      * Emitted when the user presses the return key.
      *
      *  The argument is the current text.  Note that this
      * signal is @em not emitted if the widget's @p EchoMode is set to
-     * @ref QLineEdit::Password.
+     * QLineEdit::EchoMode.
      */
     void returnPressed( const QString& );
 
@@ -298,16 +341,21 @@ signals:
      */
     void completion( const QString& );
 
+    /**
+     * Emitted when the shortcut for substring completion is pressed.
+     */
+    void substringCompletion( const QString& );
+
     /**
      * Emitted when the text rotation key-bindings are pressed.
      *
      * The argument indicates which key-binding was pressed.
-     * In @ref KLineEdit's case this can be either one of two values:
-     * @ref PrevCompletionMatch or @ref NextCompletionMatch. See
-     * @ref KCompletionBase::setKeyBinding for details.
+     * In KLineEdit's case this can be either one of two values:
+     * PrevCompletionMatch or NextCompletionMatch. See
+     * KCompletionBase::setKeyBinding for details.
      *
      * Note that this signal is @em not emitted if the completion
-     * mode is set to @p CompletionNone or @p EchoMode is @em not  normal.
+     * mode is set to @p KGlobalSettings::CompletionNone or @p echoMode() is @em not  normal.
      */
     void textRotation( KCompletionBase::KeyBindingType );
 
@@ -327,14 +375,19 @@ signals:
      * provided through since it is created and deleted
      * on demand.
      *
-     * @param the context menu about to be displayed
+     * @param p the context menu about to be displayed
      */
-    void aboutToShowContextMenu( QPopupMenu* );
+    void aboutToShowContextMenu( QPopupMenu * p );
 
 public slots:
 
     /**
-     * Iterate through all possible matches of the completed text or
+     * Re-implemented for internal reasons. API not changed.
+     */
+    virtual void setReadOnly(bool);
+
+    /**
+     * Iterates through all possible matches of the completed text or
      * the history list.
      *
      * This function simply iterates over all possible matches in case
@@ -343,23 +396,56 @@ public slots:
      *
      * @param type The key-binding invoked.
      */
-    void rotateText( KCompletionBase::KeyBindingType /* type */ );
+    void rotateText( KCompletionBase::KeyBindingType type );
 
     /**
-     * See @ref KCompletionBase::setCompletionText.
+     * See KCompletionBase::setCompletedText.
      */
     virtual void setCompletedText( const QString& );
 
     /**
-     * Sets @p items into the completion-box if @ref completionMode() is
-     * CompletionPopup. The popup will be shown immediately.
+     * Sets @p items into the completion-box if completionMode() is
+     * CompletionPopup. The popup will be shown immediately. 
+     *
+     * @param items list of completion matches to be shown in the completion box.
      */
     void setCompletedItems( const QStringList& items );
+    
+    /**
+     * Same as the above function except it allows you to temporarily
+     * turn off text completion in CompletionPopupAuto mode.
+     *
+     * TODO: Merge with above function in KDE 4.
+     * TODO: Does that make this or the above @deprecated ?
+     *
+     * @param items list of completion matches to be shown in the completion box.
+     * @param autoSuggest true if you want automatic text completion (suggestion) enabled.
+     */
+    void setCompletedItems( const QStringList& items, bool autoSuggest );
 
+    /**
+     * Reimplemented to workaround a buggy QLineEdit::clear()
+     * (changing the clipboard to the text we just had in the lineedit)
+     */
+    virtual void clear();
+
+    /**
+     * Squeezes @p text into the line edit.
+     * This can only be used with read-only line-edits.
+     * @since 3.1
+     */
+    void setSqueezedText( const QString &text);
+
+    /**
+     * Re-implemented to enable text squeezing. API is not affected.
+     */
+    virtual void setText ( const QString& );
+    
+    
 protected slots:
 
     /**
-    * Complete the remaining text with a matching one from
+    * Completes the remaining text with a matching one from
     * a given list.
     */
     virtual void makeCompletion( const QString& );
@@ -373,27 +459,62 @@ protected slots:
      * @deprecated.  Will be removed in the next major release!
      */
     void slotCancelled() {}
+    
+    /**
+     * Resets the current displayed text. 
+     * Call this function to revert a text completion if the user 
+     * cancels the request. Mostly applies to popup completions.
+     */
+    void userCancelled(const QString & cancelText);
 
 protected:
 
     /**
     * Re-implemented for internal reasons.  API not affected.
     *
-    * See @ref QLineEdit::keyPressEvent().
+    * See QLineEdit::resizeEvent().
+    */
+    virtual void resizeEvent( QResizeEvent * );
+
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::keyPressEvent().
     */
     virtual void keyPressEvent( QKeyEvent * );
 
     /**
     * Re-implemented for internal reasons.  API not affected.
     *
-    * See @ref QLineEdit::mousePressEvent().
+    * See QLineEdit::mousePressEvent().
     */
     virtual void mousePressEvent( QMouseEvent * );
 
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QWidget::mouseDoubleClickEvent().
+    */
+    virtual void mouseDoubleClickEvent( QMouseEvent * );
+
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::contextMenuEvent().
+    */
+    virtual void contextMenuEvent( QContextMenuEvent * );
+
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::createPopupMenu().
+    */
+    virtual QPopupMenu *createPopupMenu();
+
     /**
     * Re-implemented to handle URI drops.
     *
-    * See @ref QLineEdit::dropEvent().
+    * See QLineEdit::dropEvent().
     */
     virtual void dropEvent( QDropEvent * );
 
@@ -407,48 +528,68 @@ protected:
     */
     virtual void setCompletedText( const QString& /*text*/, bool /*marked*/ );
 
+
+    /**
+     * Sets the widget in userSelection mode or in automatic completion
+     * selection mode. This changes the colors of selections.
+     */
+    void setUserSelection( bool userSelection );
+
     /**
      * Reimplemented for internal reasons, the API is not affected.
      */
     virtual void create( WId = 0, bool initializeWindow = true,
                          bool destroyOldWindow = true );
 
+private slots:
+    void completionMenuActivated( int id );
+    void tripleClickTimeout();  // resets possibleTripleClick
+    void slotRestoreSelectionColors();
+    void setTextWorkaround( const QString& text );
+
 private:
+
     // Constants that represent the ID's of the popup menu.
-    // TODO: See if we can replace this mess with KActionMenu
-    // in the future though it's working lovely.
-    enum MenuID {
-        Default=0,
-        Cut,
-        Copy,
-        Paste,
-        Clear,
-        Unselect,
-        SelectAll,
+    enum MenuID
+    {
+        Default = 42,
         NoCompletion,
         AutoCompletion,
         ShellCompletion,
         PopupCompletion,
-        SemiAutoCompletion
+        ShortAutoCompletion,
+        PopupAutoCompletion
     };
 
-    /** Initializes variables.  Called from the constructors. */
+    /**
+     * Initializes variables.  Called from the constructors.
+     */
     void init();
-    /** Creates the completion box */
-    void makeCompletionBox();
-    /** Returns the context menu, creates a new one if did not exist. */
-    QPopupMenu* contextMenuInternal();
-    /** initializes the context menu */
-    void initPopup();
 
     /**
-     * Temporary functions to delete words back and foward until
-     * alternatives are available in QT3 (Seth Chaiklin, 21 may 2001)
+     * Creates the completion box
      */
-    void deleteWordBack();
-    void deleteWordForward();
+    void makeCompletionBox();
+
+    /**
+     * Checks whether we should/should not consume a key used as
+     * an accelerator.
+     */
+    bool overrideAccel (const QKeyEvent* e);
+
+    /**
+     * Properly sets the squeezed text whenever the widget is 
+     * created or resized.
+     */
+    void setSqueezedText ();
 
     bool m_bEnableMenu;
+
+    bool possibleTripleClick;  // set in mousePressEvent, deleted in tripleClickTimeout
+
+protected:
+    virtual void virtual_hook( int id, void* data );
+private:
     class KLineEditPrivate;
     KLineEditPrivate *d;
 };
