@@ -5,6 +5,7 @@
      (C) 1997-2000 Sven Radej (radej@kde.org)
      (C) 1997-2000 Matthias Ettrich (ettrich@kde.org)
      (C) 1999 Chris Schlaeger (cs@kde.org)
+     (C) 2002 Joseph Wenninger (jowenn@kde.org)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -20,25 +21,36 @@
      the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
  */
+#include "config.h"
 
 #include "kmainwindow.h"
-
+#include "kmainwindowiface.h"
+#include "ktoolbarhandler.h"
 #include <qsessionmanager.h>
 #include <qobjectlist.h>
+#include <qstyle.h>
+#include <qlayout.h>
+#include <qwidgetlist.h>
+#include <qtimer.h>
 
 #include <kaccel.h>
-#include <kapp.h>
+#include <kaction.h>
+#include <kapplication.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <khelpmenu.h>
 #include <kmenubar.h>
 #include <kstatusbar.h>
-#include <qlayout.h>
+#include <kwin.h>
+#include <kedittoolbar.h>
+#include <kmainwindow.h>
 
 #include <klocale.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <kstaticdeleter.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <netwm.h>
+#endif
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -52,11 +64,18 @@ public:
     bool settingsDirty:1;
     bool autoSaveWindowSize:1;
     bool care_about_geometry:1;
+    bool shuttingDown:1;
     QString autoSaveGroup;
     KAccel * kaccel;
+    KMainWindowInterface *m_interface;
+    KDEPrivate::ToolBarHandler *toolBarHandler;
+    QTimer* settingsTimer;
+    KToggleAction *showStatusBarAction;
+    QRect defaultWindowSize;
+    QPtrList<QDockWindow> hiddenDockWindows;
 };
 
-QList<KMainWindow>* KMainWindow::memberList = 0L;
+QPtrList<KMainWindow>* KMainWindow::memberList = 0L;
 static bool no_query_exit = false;
 static KMWSessionManaged* ksm = 0;
 static KStaticDeleter<KMWSessionManaged> ksmd;
@@ -75,35 +94,35 @@ public:
         KConfig* config = KApplication::kApplication()->sessionConfig();
         if ( KMainWindow::memberList->first() ){
             // According to Jochen Wilhelmy <digisnap@cs.tu-berlin.de>, this
-            // hook is usefull for better document orientation
+            // hook is useful for better document orientation
             KMainWindow::memberList->first()->saveGlobalProperties(config);
         }
 
-        QListIterator<KMainWindow> it(*KMainWindow::memberList);
+        QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
         int n = 0;
-        config->setGroup(QString::fromLatin1("Number"));
-        config->writeEntry(QString::fromLatin1("NumberOfWindows"), KMainWindow::memberList->count());
         for (it.toFirst(); it.current(); ++it){
             n++;
             it.current()->savePropertiesInternal(config, n);
         }
-        return TRUE;
+        config->setGroup(QString::fromLatin1("Number"));
+        config->writeEntry(QString::fromLatin1("NumberOfWindows"), n );
+        return true;
     }
 
     bool commitData( QSessionManager& sm )
     {
         // not really a fast method but the only compatible one
         if ( sm.allowsInteraction() ) {
-            bool cancelled = false;
-            QListIterator<KMainWindow> it(*KMainWindow::memberList);
+            bool canceled = false;
+            QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
             ::no_query_exit = true;
-            for (it.toFirst(); it.current() && !cancelled;){
+            for (it.toFirst(); it.current() && !canceled;){
                 KMainWindow *window = *it;
                 ++it; // Update now, the current window might get deleted
                 if ( !window->testWState( Qt::WState_ForceHide ) ) {
                     QCloseEvent e;
                     QApplication::sendEvent( window, &e );
-                    cancelled = !e.isAccepted();
+                    canceled = !e.isAccepted();
 		    /* Don't even think_about deleting widgets with
 		     Qt::WDestructiveClose flag set at this point. We
 		     are faking a close event, but we are *not*_
@@ -121,11 +140,11 @@ public:
                 }
             }
             ::no_query_exit = false;
-            if (cancelled)
+            if (canceled)
                return false;
 
             KMainWindow* last = 0;
-            for (it.toFirst(); it.current() && !cancelled; ++it){
+            for (it.toFirst(); it.current() && !canceled; ++it){
                 KMainWindow *window = *it;
                 if ( !window->testWState( Qt::WState_ForceHide ) ) {
                     last = window;
@@ -138,39 +157,78 @@ public:
         }
 
         // the user wants it, the user gets it
-        return TRUE;
+        return true;
     }
 };
 
-static bool beeing_first = true;
+static bool being_first = true;
 
 KMainWindow::KMainWindow( QWidget* parent, const char *name, WFlags f )
     : QMainWindow( parent, name, f ), KXMLGUIBuilder( this ), helpMenu2( 0 ), factory_( 0 )
 {
-    setDockMenuEnabled( FALSE );
+    initKMainWindow(name, 0);
+}
+
+KMainWindow::KMainWindow( int cflags, QWidget* parent, const char *name, WFlags f )
+    : QMainWindow( parent, name, f ), KXMLGUIBuilder( this ), helpMenu2( 0 ), factory_( 0 )
+{
+    initKMainWindow(name, cflags);
+}
+
+void KMainWindow::initKMainWindow(const char *name, int cflags)
+{
+    setDockMenuEnabled( false );
     mHelpMenu = 0;
     kapp->setTopWidget( this );
+    actionCollection()->setWidget( this );
     connect(kapp, SIGNAL(shutDown()), this, SLOT(shuttingDown()));
     if( !memberList )
-        memberList = new QList<KMainWindow>;
+        memberList = new QPtrList<KMainWindow>;
 
     if ( !ksm )
-        ksm = ksmd.setObject(new KMWSessionManaged());
-    if ( !name ) {
-        // set a unique object name. Required by session management.
-        QCString s;
-        int unusedNumber= 0;
-        KMainWindow *existingWin;
-        do {
-            s.setNum( ++unusedNumber );
-            s= kapp->instanceName() + "-mainwindow#" + s;
-            for ( existingWin= memberList->first(); existingWin!=0; 
-                  existingWin= memberList->next() )
-                if ( existingWin->name() == s )
-                    break;
-        } while ( existingWin!=0 );
-        setName( s );
+        ksm = ksmd.setObject(ksm, new KMWSessionManaged());
+    // set a unique object name. Required by session management.
+    QCString objname;
+    QCString s;
+    int unusedNumber;
+    if ( !name )
+        { // no name given
+        objname = kapp->instanceName() + "-mainwindow#";
+        s = objname + '1'; // start adding number immediately
+        unusedNumber = 1;
+        }
+    else if( name[ strlen( name ) - 1 ] == '#' )
+        { // trailing # - always add a number
+        objname = name;
+        s = objname + '1'; // start adding number immediately
+        unusedNumber = 1;
+        }
+    else
+        {
+        objname = name;
+        s = objname;
+        unusedNumber = 0; // add numbers only when needed
+        }
+    for(;;) {
+        QWidgetList* list = kapp->topLevelWidgets();
+        QWidgetListIt it( *list );
+        bool found = false;
+        for( QWidget* w = it.current();
+             w != NULL;
+             ++it, w = it.current())
+            if( w != this && w->name() == s )
+                {
+                found = true;
+                break;
+                }
+        delete list;
+        if( !found )
+            break;
+        s.setNum( ++unusedNumber );
+        s = objname + s;
     }
+    setName( s );
+
     memberList->append( this );
 
     d = new KMainWindowPrivate;
@@ -178,9 +236,13 @@ KMainWindow::KMainWindow( QWidget* parent, const char *name, WFlags f )
     d->settingsDirty = false;
     d->autoSaveSettings = false;
     d->autoSaveWindowSize = true; // for compatibility
-    d->kaccel = 0L;
-    if ((d->care_about_geometry = beeing_first)) {
-        beeing_first = false;
+    d->kaccel = actionCollection()->kaccel();
+    d->toolBarHandler = 0;
+    d->settingsTimer = 0;
+    d->showStatusBarAction = NULL;
+    d->shuttingDown = false;
+    if ((d->care_about_geometry = being_first)) {
+        being_first = false;
         if ( kapp->geometryArgument().isNull() ) // if there is no geometry, it doesn't mater
             d->care_about_geometry = false;
         else
@@ -188,6 +250,28 @@ KMainWindow::KMainWindow( QWidget* parent, const char *name, WFlags f )
     }
 
     setCaption( kapp->caption() );
+    if ( cflags & NoDCOPObject)
+        d->m_interface = 0;
+    else
+        d->m_interface = new KMainWindowInterface(this);
+
+    if (!kapp->authorize("movable_toolbars"))
+        setDockWindowsMovable(false);
+}
+
+KAction *KMainWindow::toolBarMenuAction()
+{
+    if ( !d->toolBarHandler )
+	return 0;
+
+    return d->toolBarHandler->toolBarMenuAction();
+}
+
+
+void KMainWindow::setupToolbarMenuActions()
+{
+    if ( d->toolBarHandler )
+        d->toolBarHandler->setupActions();
 }
 
 void KMainWindow::parseGeometry(bool parsewidth)
@@ -195,6 +279,9 @@ void KMainWindow::parseGeometry(bool parsewidth)
     assert ( !kapp->geometryArgument().isNull() );
     assert ( d->care_about_geometry );
 
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
+    // FIXME: (E) Implement something similar for Qt Embedded (or decide we don't need it)
     int x, y;
     int w, h;
     int m = XParseGeometry( kapp->geometryArgument().latin1(), &x, &y, (unsigned int*)&w, (unsigned int*)&h);
@@ -221,12 +308,15 @@ void KMainWindow::parseGeometry(bool parsewidth)
             y = KApplication::desktop()->height() + y - h;
         move(x, y);
     }
+#endif
 }
 
 KMainWindow::~KMainWindow()
 {
+    delete d->settingsTimer;
     QMenuBar* mb = internalMenuBar();
     delete mb;
+    delete d->m_interface;
     delete d;
     memberList->remove( this );
 }
@@ -262,10 +352,10 @@ KPopupMenu* KMainWindow::customHelpMenu( bool showWhatsThis )
 bool KMainWindow::canBeRestored( int number )
 {
     if ( !kapp->isRestored() )
-        return FALSE;
+        return false;
     KConfig *config = kapp->sessionConfig();
     if ( !config )
-        return FALSE;
+        return false;
     config->setGroup( QString::fromLatin1("Number") );
     int n = config->readNumEntry( QString::fromLatin1("NumberOfWindows") , 1 );
     return number >= 1 && number <= n;
@@ -288,17 +378,47 @@ const QString KMainWindow::classNameOfToplevel( int number )
         return config->readEntry( QString::fromLatin1("ClassName") );
 }
 
+void KMainWindow::show()
+{
+    QMainWindow::show();
+
+    for ( QPtrListIterator<QDockWindow> it( d->hiddenDockWindows ); it.current(); ++it )
+	it.current()->show();
+
+    d->hiddenDockWindows.clear();
+}
+
+void KMainWindow::hide()
+{
+    if ( isVisible() ) {
+
+        d->hiddenDockWindows.clear();
+
+        QObjectList *list = queryList( "QDockWindow" );
+        for( QObjectListIt it( *list ); it.current(); ++it ) {
+            QDockWindow *dw = (QDockWindow*)it.current();
+            if ( dw->isTopLevel() && dw->isVisible() ) {
+                d->hiddenDockWindows.append( dw );
+                dw->hide();
+            }
+        }
+        delete list;
+    }
+
+    QWidget::hide();
+}
+
 bool KMainWindow::restore( int number, bool show )
 {
     if ( !canBeRestored( number ) )
-        return FALSE;
+        return false;
     KConfig *config = kapp->sessionConfig();
     if ( readPropertiesInternal( config, number ) ){
         if ( show )
             KMainWindow::show();
-        return FALSE;
+        return false;
     }
-    return FALSE;
+    return false;
 }
 
 KXMLGUIFactory *KMainWindow::guiFactory()
@@ -308,6 +428,52 @@ KXMLGUIFactory *KMainWindow::guiFactory()
     return factory_;
 }
 
+int KMainWindow::configureToolbars()
+{
+    saveMainWindowSettings(KGlobal::config());
+    KEditToolbar dlg(actionCollection(), xmlFile(), true, this);
+    connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(saveNewToolbarConfig()));
+    return dlg.exec();
+}
+
+void KMainWindow::saveNewToolbarConfig()
+{
+    createGUI(xmlFile());
+    applyMainWindowSettings( KGlobal::config() );
+}
+
+void KMainWindow::setupGUI( int options, const QString & xmlfile ) {
+    if( options & Keys ){
+        KStdAction::keyBindings(guiFactory(),
+                    SLOT(configureShortcuts()), actionCollection());
+    }
+
+    if( (options & StatusBar) && internalStatusBar() ){
+        createStandardStatusBarAction();
+    }
+
+    if( options & ToolBar ){
+        setStandardToolBarMenuEnabled( true );
+        KStdAction::configureToolbars(this,
+                      SLOT(configureToolbars() ), actionCollection());
+    }
+
+    if( options & Create ){
+        createGUI(xmlfile);
+    }
+
+    if( options & Save ){
+        // setupGUI() is typically called in the constructor before show(),
+        // so the default window size will be incorrect unless the application
+        // hard coded the size which they should try not to do (i.e. use
+        // size hints).
+        if(!isShown())
+          adjustSize();
+        setAutoSaveSettings();
+    }
+
+}
+
 void KMainWindow::createGUI( const QString &xmlfile, bool _conserveMemory )
 {
     // disabling the updates prevents unnecessary redraws
@@ -370,17 +536,18 @@ void KMainWindow::createGUI( const QString &xmlfile, bool _conserveMemory )
 
       QDomDocument doc = domDocument();
 
-      QDomElement e = doc.documentElement().firstChild().toElement();
-      for (; !e.isNull(); e = e.nextSibling().toElement() ) {
+      for( QDomNode n = doc.documentElement().firstChild();
+           !n.isNull(); n = n.nextSibling())
+      {
+          QDomElement e = n.toElement();
+
           if ( e.tagName().lower() == "toolbar" )
               factory_->resetContainer( e.attribute( "name" ) );
           else if ( e.tagName().lower() == "menubar" )
               factory_->resetContainer( e.tagName(), true );
       }
 
-      // ### We cannot free the DOM document here, because then the toolbar state saving
-      // doesn't work (Simon)
-//      conserveMemory();
+      conserveMemory();
     }
 
     setUpdatesEnabled( true );
@@ -410,8 +577,11 @@ void KMainWindow::setCaption( const QString &caption, bool modified )
 void KMainWindow::setPlainCaption( const QString &caption )
 {
     QMainWindow::setCaption( caption );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     NETWinInfo info( qt_xdisplay(), winId(), qt_xrootwin(), 0 );
     info.setName( caption.utf8().data() );
+#endif
 }
 
 void KMainWindow::appHelpActivated( void )
@@ -424,33 +594,52 @@ void KMainWindow::appHelpActivated( void )
     mHelpMenu->appHelpActivated();
 }
 
+void KMainWindow::slotStateChanged(const QString &newstate)
+{
+  stateChanged(newstate, KXMLGUIClient::StateNoReverse);
+}
+
+/*
+ * Get rid of this for KDE 4.0
+ */
+void KMainWindow::slotStateChanged(const QString &newstate,
+                                   KXMLGUIClient::ReverseStateChange reverse)
+{
+  stateChanged(newstate, reverse);
+}
+
+/*
+ * Enable this for KDE 4.0
+ */
+// void KMainWindow::slotStateChanged(const QString &newstate,
+//                                    bool reverse)
+// {
+//   stateChanged(newstate,
+//                reverse ? KXMLGUIClient::StateReverse : KXMLGUIClient::StateNoReverse);
+// }
 
 void KMainWindow::closeEvent ( QCloseEvent *e )
 {
+    // Save settings if auto-save is enabled, and settings have changed
+    if (d->settingsDirty && d->autoSaveSettings)
+        saveAutoSaveSettings();
+
     if (queryClose()) {
         e->accept();
 
-        // Save settings if auto-save is enabled, and settings have changed
-        if (d->settingsDirty && d->autoSaveSettings)
-        {
-            //kdDebug(200) << "KMainWindow::closeEvent -> saving settings" << endl;
-            saveMainWindowSettings( KGlobal::config(), d->autoSaveGroup );
-            KGlobal::config()->sync();
-            d->settingsDirty = false;
-        }
-
         int not_withdrawn = 0;
-        QListIterator<KMainWindow> it(*KMainWindow::memberList);
+        QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
         for (it.toFirst(); it.current(); ++it){
             if ( !it.current()->isHidden() && it.current()->isTopLevel() && it.current() != this )
                 not_withdrawn++;
         }
 
         if ( !no_query_exit && not_withdrawn <= 0 ) { // last window close accepted?
-            if ( queryExit() ) {            // Yes, Quit app?
-                // We saved the toolbars already
+            if ( queryExit() && !kapp->sessionSaving() && !d->shuttingDown ) { // Yes, Quit app?
+                // don't call queryExit() twice
                 disconnect(kapp, SIGNAL(shutDown()), this, SLOT(shuttingDown()));
-                kapp->deref();             // ...and quit aplication.
+                d->shuttingDown = true;
+                kapp->deref();             // ...and quit application.
             }  else {
                 // cancel closing, it's stupid to end up with no windows at all....
                 e->ignore();
@@ -461,12 +650,12 @@ void KMainWindow::closeEvent ( QCloseEvent *e )
 
 bool KMainWindow::queryExit()
 {
-    return TRUE;
+    return true;
 }
 
 bool KMainWindow::queryClose()
 {
-    return TRUE;
+    return true;
 }
 
 void KMainWindow::saveGlobalProperties( KConfig*  )
@@ -483,7 +672,7 @@ void KMainWindow::updateRects()
 }
 #endif
 
-void KMainWindow::showAboutApplication( void )
+void KMainWindow::showAboutApplication()
 {
 }
 
@@ -514,11 +703,13 @@ void KMainWindow::savePropertiesInternal( KConfig *config, int number )
 void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configGroup)
 {
     kdDebug(200) << "KMainWindow::saveMainWindowSettings " << configGroup << endl;
-    QString entry;
-    QStrList entryList;
+    QString oldGroup;
 
     if (!configGroup.isEmpty())
+    {
+       oldGroup = config->group();
        config->setGroup(configGroup);
+    }
 
     // Called by session management - or if we want to save the window size anyway
     if ( d->autoSaveWindowSize )
@@ -526,28 +717,24 @@ void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configG
 
     QStatusBar* sb = internalStatusBar();
     if (sb) {
-        entryList.clear();
-        if ( sb->isHidden() )
-            entryList.append("Disabled");
-        else
-            entryList.append("Enabled");
-        config->writeEntry(QString::fromLatin1("StatusBar"), entryList, ';');
+       if(!config->hasDefault("StatusBar") && !sb->isHidden() )
+           config->revertToDefault("StatusBar");
+       else
+           config->writeEntry("StatusBar", sb->isHidden() ? "Disabled" : "Enabled");
     }
 
     QMenuBar* mb = internalMenuBar();
     if (mb) {
-        entryList.clear();
-        if ( mb->isHidden() )
-            entryList.append("Disabled");
-        else
-            entryList.append("Enabled");
-        config->writeEntry(QString::fromLatin1("MenuBar"), entryList, ';');
+       QString MenuBar = QString::fromLatin1("MenuBar");
+       if(!config->hasDefault("MenuBar") && !mb->isHidden() )
+           config->revertToDefault("MenuBar");
+       else
+           config->writeEntry("MenuBar", mb->isHidden() ? "Disabled" : "Enabled");
     }
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     KToolBar *toolbar = 0;
-    QString toolKey;
-    QListIterator<KToolBar> it( toolBarIterator() );
+    QPtrListIterator<KToolBar> it( toolBarIterator() );
     while ( ( toolbar = it.current() ) ) {
         ++it;
         QString group;
@@ -555,13 +742,51 @@ void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configG
         {
            // Give a number to the toolbar, but prefer a name if there is one,
            // because there's no real guarantee on the ordering of toolbars
-           group = (!strcmp(toolbar->name(), "unnamed") ? QString::number(n) : QString(" ")+toolbar->name());
+           group = (!::qstrcmp(toolbar->name(), "unnamed") ? QString::number(n) : QString(" ")+toolbar->name());
            group.prepend(" Toolbar");
            group.prepend(configGroup);
         }
         toolbar->saveSettings(config, group);
         n++;
     }
+    if (!configGroup.isEmpty())
+       config->setGroup(oldGroup);
+}
+
+void KMainWindow::setStandardToolBarMenuEnabled( bool enable )
+{
+    if ( enable ) {
+        if ( d->toolBarHandler )
+            return;
+
+    d->toolBarHandler = new KDEPrivate::ToolBarHandler( this );
+
+    if ( factory() )
+        factory()->addClient( d->toolBarHandler );
+    } else {
+        if ( !d->toolBarHandler )
+            return;
+
+        if ( factory() )
+            factory()->removeClient( d->toolBarHandler );
+
+        delete d->toolBarHandler;
+        d->toolBarHandler = 0;
+    }
+}
+
+bool KMainWindow::isStandardToolBarMenuEnabled() const
+{
+    return ( d->toolBarHandler != 0 );
+}
+
+void KMainWindow::createStandardStatusBarAction(){
+  if(!d->showStatusBarAction){
+    d->showStatusBarAction = KStdAction::showStatusbar(this, SLOT(setSettingsDirty()), actionCollection());
+    KStatusBar *sb = statusBar(); // Creates statusbar if it doesn't exist already.
+    connect(d->showStatusBarAction, SIGNAL(toggled(bool)), sb, SLOT(setShown(bool)));
+    d->showStatusBarAction->setChecked(sb->isHidden());
+  }
 }
 
 bool KMainWindow::readPropertiesInternal( KConfig *config, int number )
@@ -590,11 +815,12 @@ bool KMainWindow::readPropertiesInternal( KConfig *config, int number )
 
 void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &configGroup)
 {
-    kdDebug(200) << "KMainWindow::applyMainWindowSettings" << endl;
-    QString entry;
-    QStrList entryList;
-    int i = 0; // Number of entries in list
+    return applyMainWindowSettings(config,configGroup,false);
+}
 
+void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &configGroup,bool force)
+{
+    kdDebug(200) << "KMainWindow::applyMainWindowSettings" << endl;
     if (!configGroup.isEmpty())
        config->setGroup(configGroup);
 
@@ -602,30 +828,27 @@ void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &config
 
     QStatusBar* sb = internalStatusBar();
     if (sb) {
-        entryList.clear();
-        i = config->readListEntry (QString::fromLatin1("StatusBar"), entryList, ';');
-        entry = entryList.first();
-        if (entry == QString::fromLatin1("Disabled"))
-            sb->hide();
+        QString entry = config->readEntry("StatusBar", "Enabled");
+        if ( entry == "Disabled" )
+           sb->hide();
         else
-            sb->show();
+           sb->show();
+        if(d->showStatusBarAction)
+            d->showStatusBarAction->setChecked(!sb->isHidden());
     }
 
     QMenuBar* mb = internalMenuBar();
     if (mb) {
-        entryList.clear();
-        i = config->readListEntry (QString::fromLatin1("MenuBar"), entryList, ';');
-        entry = entryList.first();
-        if (entry==QString::fromLatin1("Disabled"))
-            mb->hide();
+        QString entry = config->readEntry ("MenuBar", "Enabled");
+        if ( entry == "Disabled" )
+           mb->hide();
         else
-            mb->show();
+           mb->show();
     }
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     KToolBar *toolbar;
-    QString toolKey;
-    QListIterator<KToolBar> it( toolBarIterator() ); // must use own iterator
+    QPtrListIterator<KToolBar> it( toolBarIterator() ); // must use own iterator
 
     for ( ; it.current(); ++it) {
         toolbar= it.current();
@@ -634,11 +857,11 @@ void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &config
         {
            // Give a number to the toolbar, but prefer a name if there is one,
            // because there's no real guarantee on the ordering of toolbars
-           group = (!strcmp(toolbar->name(), "unnamed") ? QString::number(n) : QString(" ")+toolbar->name());
+           group = (!::qstrcmp(toolbar->name(), "unnamed") ? QString::number(n) : QString(" ")+toolbar->name());
            group.prepend(" Toolbar");
            group.prepend(configGroup);
         }
-        toolbar->applySettings(config, group);
+        toolbar->applySettings(config, group, force);
         n++;
     }
 
@@ -655,18 +878,41 @@ void KMainWindow::finalizeGUI( bool force )
     // we call positionYourself again for each of them, but this time
     // the toolbariterator should give them in the proper order.
     // Both the XMLGUI and applySettings call this, hence "force" for the latter.
-    QListIterator<KToolBar> it( toolBarIterator() );
+    QPtrListIterator<KToolBar> it( toolBarIterator() );
     for ( ; it.current() ; ++ it )
-        it.current()->positionYourself( force );
+            it.current()->positionYourself( force );
 
     d->settingsDirty = false;
 }
 
 void KMainWindow::saveWindowSize( KConfig * config ) const
 {
-    QWidget *desk = KApplication::desktop();
-    config->writeEntry(QString::fromLatin1("Width %1").arg(desk->width()), width() );
-    config->writeEntry(QString::fromLatin1("Height %1").arg(desk->height()), height() );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+  int scnum = QApplication::desktop()->screenNumber(parentWidget());
+  QRect desk = QApplication::desktop()->screenGeometry(scnum);
+  // save maximalization as desktop size + 1 in that direction
+  KWin::WindowInfo info = KWin::windowInfo( winId(), NET::WMState );
+  int w = info.state() & NET::MaxHoriz ? desk.width() + 1 : width();
+  int h = info.state() & NET::MaxVert ? desk.height() + 1 : height();
+  QRect size( desk.width(), w, desk.height(), h );
+#else
+  int w = 500;
+  int h = 500;
+  QRect desk( 100, 100, 200, 200 ); // fixme
+  QRect size( 100, 100, 200, 200 ); // fixme
+#endif
+  bool defaultSize = (size == d->defaultWindowSize);
+  QString widthString = QString::fromLatin1("Width %1").arg(desk.width());
+  QString heightString = QString::fromLatin1("Height %1").arg(desk.height());
+  if (!config->hasDefault(widthString) && defaultSize)
+     config->revertToDefault(widthString);
+  else
+     config->writeEntry(widthString, w );
+
+  if (!config->hasDefault(heightString) && defaultSize)
+     config->revertToDefault(heightString);
+  else
+     config->writeEntry(heightString, h );
 }
 
 void KMainWindow::restoreWindowSize( KConfig * config )
@@ -675,9 +921,12 @@ void KMainWindow::restoreWindowSize( KConfig * config )
         parseGeometry(true);
     } else {
         // restore the size
-        QWidget *desk = KApplication::desktop();
-        QSize size( config->readNumEntry( QString::fromLatin1("Width %1").arg(desk->width()), 0 ),
-                    config->readNumEntry( QString::fromLatin1("Height %1").arg(desk->height()), 0 ) );
+        int scnum = QApplication::desktop()->screenNumber(parentWidget());
+        QRect desk = QApplication::desktop()->screenGeometry(scnum);
+        if ( d->defaultWindowSize.isNull() ) // only once
+          d->defaultWindowSize = QRect(desk.width(), width(), desk.height(), height()); // store default values
+        QSize size( config->readNumEntry( QString::fromLatin1("Width %1").arg(desk.width()), 0 ),
+                    config->readNumEntry( QString::fromLatin1("Height %1").arg(desk.height()), 0 ) );
         if (size.isEmpty()) {
             // try the KDE 2.0 way
             size = QSize( config->readNumEntry( QString::fromLatin1("Width"), 0 ),
@@ -688,8 +937,20 @@ void KMainWindow::restoreWindowSize( KConfig * config )
                 config->writeEntry( QString::fromLatin1("Height"), 0 );
             }
         }
-        if ( !size.isEmpty() )
-            resize( size );
+        if ( !size.isEmpty() ) {
+            int state = ( size.width() > desk.width() ? NET::MaxHoriz : 0 )
+                        | ( size.height() > desk.height() ? NET::MaxVert : 0 );
+            if(( state & NET::Max ) == NET::Max )
+                ; // no resize
+            else if(( state & NET::MaxHoriz ) == NET::MaxHoriz )
+                resize( width(), size.height());
+            else if(( state & NET::MaxVert ) == NET::MaxVert )
+                resize( size.width(), height());
+            else
+                resize( size );
+            // QWidget::showMaximized() is both insufficient and broken
+            KWin::setState( winId(), state );
+        }
     }
 }
 
@@ -698,10 +959,26 @@ bool KMainWindow::initialGeometrySet() const
     return d->care_about_geometry;
 }
 
+void KMainWindow::ignoreInitialGeometry()
+{
+    d->care_about_geometry = false;
+}
+
 void KMainWindow::setSettingsDirty()
 {
     //kdDebug(200) << "KMainWindow::setSettingsDirty" << endl;
     d->settingsDirty = true;
+    if ( d->autoSaveSettings )
+    {
+        // Use a timer to save "immediately" user-wise, but not too immediately
+        // (to compress calls and save only once, in case of multiple changes)
+        if ( !d->settingsTimer )
+        {
+           d->settingsTimer = new QTimer( this );
+           connect( d->settingsTimer, SIGNAL( timeout() ), SLOT( saveAutoSaveSettings() ) );
+        }
+        d->settingsTimer->start( 500, true );
+    }
 }
 
 bool KMainWindow::settingsDirty() const
@@ -720,8 +997,9 @@ void KMainWindow::setAutoSaveSettings( const QString & groupName, bool saveWindo
     d->autoSaveGroup = groupName;
     d->autoSaveWindowSize = saveWindowSize;
     // Get notified when the user moves a toolbar around
-    connect( this, SIGNAL( endMovingToolBar( QToolBar * ) ),
+    connect( this, SIGNAL( dockWindowPositionChanged( QDockWindow * ) ),
              this, SLOT( setSettingsDirty() ) );
+
     // Now read the previously saved settings
     applyMainWindowSettings( KGlobal::config(), groupName );
 }
@@ -729,6 +1007,29 @@ void KMainWindow::setAutoSaveSettings( const QString & groupName, bool saveWindo
 void KMainWindow::resetAutoSaveSettings()
 {
     d->autoSaveSettings = false;
+    if ( d->settingsTimer )
+        d->settingsTimer->stop();
+}
+
+bool KMainWindow::autoSaveSettings() const
+{
+    return d->autoSaveSettings;
+}
+
+QString KMainWindow::autoSaveGroup() const
+{
+    return d->autoSaveGroup;
+}
+
+void KMainWindow::saveAutoSaveSettings()
+{
+    Q_ASSERT( d->autoSaveSettings );
+    //kdDebug(200) << "KMainWindow::saveAutoSaveSettings -> saving settings" << endl;
+    saveMainWindowSettings( KGlobal::config(), d->autoSaveGroup );
+    KGlobal::config()->sync();
+    d->settingsDirty = false;
+    if ( d->settingsTimer )
+        d->settingsTimer->stop();
 }
 
 void KMainWindow::resizeEvent( QResizeEvent * )
@@ -737,50 +1038,72 @@ void KMainWindow::resizeEvent( QResizeEvent * )
         setSettingsDirty();
 }
 
+bool KMainWindow::hasMenuBar()
+{
+    return (internalMenuBar());
+}
+
 KMenuBar *KMainWindow::menuBar()
 {
-    KMenuBar * mb = static_cast<KMenuBar *>(internalMenuBar());
-    if ( !mb )
+    KMenuBar * mb = internalMenuBar();
+    if ( !mb ) {
         mb = new KMenuBar( this );
+        // trigger a re-layout and trigger a call to the private
+        // setMenuBar method.
+        QMainWindow::menuBar();
+    }
     return mb;
 }
 
 KStatusBar *KMainWindow::statusBar()
 {
-    KStatusBar * sb = static_cast<KStatusBar *>(internalStatusBar());
-    if ( !sb )
+    KStatusBar * sb = internalStatusBar();
+    if ( !sb ) {
         sb = new KStatusBar( this );
+        // trigger a re-layout and trigger a call to the private
+        // setStatusBar method.
+        QMainWindow::statusBar();
+    }
     return sb;
 }
 
 void KMainWindow::shuttingDown()
 {
-    // call the virtual queryExit
-    queryExit();
+    // Needed for Qt <= 3.0.3 at least to prevent reentrancy
+    // when queryExit() shows a dialog. Check before removing!
+    static bool reentrancy_protection = false;
+    if (!reentrancy_protection)
+    {
+       reentrancy_protection = true;
+       // call the virtual queryExit
+       queryExit();
+       reentrancy_protection = false;
+    }
+
 }
 
-QMenuBar *KMainWindow::internalMenuBar()
+KMenuBar *KMainWindow::internalMenuBar()
 {
-    QObjectList *l = queryList( "QMenuBar" );
+    QObjectList *l = queryList( "KMenuBar", 0, false, false );
     if ( !l || !l->first() ) {
         delete l;
         return 0;
     }
 
-    QMenuBar *m = (QMenuBar*)l->first();
+    KMenuBar *m = (KMenuBar*)l->first();
     delete l;
     return m;
 }
 
-QStatusBar *KMainWindow::internalStatusBar()
+KStatusBar *KMainWindow::internalStatusBar()
 {
-    QObjectList *l = queryList( "QStatusBar" );
+    QObjectList *l = queryList( "KStatusBar", 0, false, false );
     if ( !l || !l->first() ) {
         delete l;
         return 0;
     }
 
-    QStatusBar *s = (QStatusBar*)l->first();
+    KStatusBar *s = (KStatusBar*)l->first();
     delete l;
     return s;
 }
@@ -802,14 +1125,14 @@ KToolBar *KMainWindow::toolBar( const char * name )
     if ( builderClient() )
         return new KToolBar(this, name, honor_mode); // XMLGUI constructor
     else
-        return new KToolBar(this, Top, false, name, honor_mode ); // non-XMLGUI
+        return new KToolBar(this, DockTop, false, name, honor_mode ); // non-XMLGUI
 }
 
-QListIterator<KToolBar> KMainWindow::toolBarIterator()
+QPtrListIterator<KToolBar> KMainWindow::toolBarIterator()
 {
     toolbarList.clear();
-    QList<QToolBar> lst;
-    for ( int i = (int)QMainWindow::Unmanaged; i <= (int)Minimized; ++i ) {
+    QPtrList<QToolBar> lst;
+    for ( int i = (int)QMainWindow::DockUnmanaged; i <= (int)DockMinimized; ++i ) {
         lst = toolBars( (ToolBarDock)i );
         for ( QToolBar *tb = lst.first(); tb; tb = lst.next() ) {
             if ( !tb->inherits( "KToolBar" ) )
@@ -817,7 +1140,7 @@ QListIterator<KToolBar> KMainWindow::toolBarIterator()
             toolbarList.append( (KToolBar*)tb );
         }
     }
-    return QListIterator<KToolBar>( toolbarList );
+    return QPtrListIterator<KToolBar>( toolbarList );
 }
 
 KAccel * KMainWindow::accel()
@@ -827,11 +1150,71 @@ KAccel * KMainWindow::accel()
     return d->kaccel;
 }
 
-void KMainWindow::paintEvent( QPaintEvent * )
+void KMainWindow::paintEvent( QPaintEvent * pe )
 {
-    // do nothing
+    QMainWindow::paintEvent(pe); //Upcall to handle SH_MainWindow_SpaceBelowMenuBar rendering
 }
 
+QSize KMainWindow::sizeForCentralWidgetSize(QSize size)
+{
+    KToolBar *tb = (KToolBar*)child( "mainToolBar", "KToolBar" );
+    if (tb && !tb->isHidden()) {
+        switch( tb->barPos() )
+        {
+          case KToolBar::Top:
+          case KToolBar::Bottom:
+            size += QSize(0, tb->sizeHint().height());
+            break;
+
+          case KToolBar::Left:
+          case KToolBar::Right:
+            size += QSize(toolBar()->sizeHint().width(), 0);
+            break;
+
+          case KToolBar::Flat:
+            size += QSize(0, 3+kapp->style().pixelMetric( QStyle::PM_DockWindowHandleExtent ));
+            break;
+
+          default:
+            break;
+        }
+    }
+    KMenuBar *mb = internalMenuBar();
+    if (mb && !mb->isHidden()) {
+        size += QSize(0,mb->heightForWidth(size.width()));
+        if (style().styleHint(QStyle::SH_MainWindow_SpaceBelowMenuBar, this))
+           size += QSize( 0, dockWindowsMovable() ? 1 : 2);
+    }
+    QStatusBar *sb = internalStatusBar();
+    if( sb && !sb->isHidden() )
+       size += QSize(0, sb->sizeHint().height());
+
+    return size;
+}
+
+#if KDE_IS_VERSION( 3, 9, 0 )
+#ifdef __GNUC__
+#warning Remove, should be in Qt
+#endif
+#endif
+void KMainWindow::setIcon( const QPixmap& p )
+{
+    QMainWindow::setIcon( p );
+    // Qt3 doesn't support _NET_WM_ICON, but KApplication::setTopWidget(), which
+    // is used by KMainWindow, sets it
+    KWin::setIcons( winId(), p, QPixmap());
+}
+
+// why do we support old gcc versions? using KXMLGUIBuilder::finalizeGUI;
+// DF: because they compile KDE much faster :)
+void KMainWindow::finalizeGUI( KXMLGUIClient *client )
+{ KXMLGUIBuilder::finalizeGUI( client ); }
+
+void KMainWindow::virtual_hook( int id, void* data )
+{ KXMLGUIBuilder::virtual_hook( id, data );
+  KXMLGUIClient::virtual_hook( id, data ); }
+
+
 
 #include "kmainwindow.moc"
 
