@@ -96,7 +96,7 @@ private:
   * construction. It will reread this config group on a
   * KApplication::appearanceChanged() signal.
   *
-  * @version $Id: ktoolbar.h,v 1.176 2003/11/05 11:43:22 tokoe Exp $
+  * @version $Id: ktoolbar.h,v 1.179 2004/07/09 09:40:15 waba Exp $
   * @author Reginald Stadlbauer <reggie@kde.org>, Stephan Kulow <coolo@kde.org>, Sven Radej <radej@kde.org>.
   */
 
@@ -199,6 +199,7 @@ public:
    * @param enabled Enable or disable the button at startup
    * @param text    The tooltip or toolbar text (depending on state)
    * @param index   The position of the button. (-1 = at end).
+   * @param _instance instance this button will belong to
    *
    * @return The item index.
    */
@@ -220,6 +221,7 @@ public:
    * @param enabled  Enable or disable the button at startup
    * @param text     The tooltip or toolbar text (depending on state)
    * @param index    The position of the button. (-1 = at end).
+   * @param _instance instance this button will belong to
    *
    * @return The item index.
    */
@@ -899,8 +901,10 @@ public:
 
   /**
    * Read the toolbar settings from group @p configGroup in @p config
-   * and apply them.
+   * and apply them. Even default settings are re-applied if @p force is set.
    */
+    void applySettings(KConfig *config, const QString &configGroup,bool force);
+    // KDE4 merge with force=false
     void applySettings(KConfig *config, const QString &configGroup);
 
   /*
@@ -1094,6 +1098,7 @@ private:
     void getAttributes( QString &position, QString &icontext, int &index );
     int dockWindowIndex();
     KPopupMenu *contextMenu();
+    void doModeChange();
 
     QMap<QWidget*, int > widget2id;
     typedef QMap<int, QWidget* > Id2WidgetMap;
