@@ -4,7 +4,7 @@
  * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2000 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
- *           (C) 2002 Apple Computer, Inc.
+ *           (C) 2002-2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +21,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_style.h,v 1.97.2.1 2004/02/29 15:28:01 mueller Exp $
  */
 #ifndef RENDERSTYLE_H
 #define RENDERSTYLE_H
@@ -191,9 +190,10 @@ enum EFloat {
 // Border attributes. Not inherited.
 
 
+// These have been defined in the order of their precedence for border-collapsing. Do
+// not change this order!
 enum EBorderStyle {
-    BNONE, BHIDDEN, DOTTED, DASHED, DOUBLE, SOLID,
-    OUTSET, INSET, GROOVE, RIDGE
+    BNONE, BHIDDEN, INSET, GROOVE, RIDGE, OUTSET, DOTTED, DASHED, SOLID, DOUBLE
 };
 
 class BorderValue
@@ -205,6 +205,16 @@ public:
     unsigned short width : 12;
     EBorderStyle style : 4;
 
+    bool nonZero() const
+    {
+      // rikkus: workaround for gcc 2.95.3
+      return width!=0 && !(style==BNONE);
+    }
+
+    bool isTransparent() const {
+        return color.isValid() && qAlpha(color.rgb()) == 0;
+    }
+
     bool operator==(const BorderValue& o) const
     {
     	return width==o.width && style==o.style && color==o.color;
@@ -212,6 +222,30 @@ public:
 
 };
 
+enum EBorderPrecedence { BOFF, BTABLE, BCOLGROUP, BCOL, BROWGROUP, BROW, BCELL };
+
+struct CollapsedBorderValue
+{
+    CollapsedBorderValue() :border(0), precedence(BOFF) {}
+    CollapsedBorderValue(const BorderValue* b, EBorderPrecedence p) :border(b), precedence(p) {}
+
+    int width() const { return border && border->nonZero() ? border->width : 0; }
+    EBorderStyle style() const { return border ? border->style : BHIDDEN; }
+    bool exists() const { return border; }
+    QColor color() const { return border ? border->color : QColor(); }
+    bool isTransparent() const { return border ? border->isTransparent() : true; }
+
+    bool operator==(const CollapsedBorderValue& o) const
+    {
+        if (!border) return !o.border;
+        if (!o.border) return false;
+        return *border == *o.border && precedence == o.precedence;
+    }
+
+    const BorderValue* border;
+    EBorderPrecedence precedence;
+};
+
 class BorderData : public Shared<BorderData>
 {
 public:
@@ -222,7 +256,7 @@ public:
 
     bool hasBorder() const
     {
-    	return left.width || right.width || top.width || bottom.width;
+    	return left.nonZero() || right.nonZero() || top.nonZero() || bottom.nonZero();
     }
 
     bool operator==(const BorderData& o) const
@@ -253,6 +287,10 @@ public:
 //------------------------------------------------
 // Box attributes. Not inherited.
 
+enum EBoxSizing {
+    BORDER_BOX, CONTENT_BOX
+};
+
 class StyleBoxData : public Shared<StyleBoxData>
 {
 public:
@@ -281,6 +319,8 @@ public:
 
     Length vertical_align;
 
+    EBoxSizing box_sizing;
+
     signed int z_index :31;
     bool z_auto        : 1;
 };
@@ -289,7 +329,7 @@ public:
 // Random visual rendering model attributes. Not inherited.
 
 enum EOverflow {
-    OVISIBLE, OHIDDEN, OSCROLL, OAUTO
+    OVISIBLE, OHIDDEN, OSCROLL, OAUTO, OMARQUEE
 };
 
 enum EVerticalAlign {
@@ -365,6 +405,53 @@ public:
     BorderValue outline;
 };
 
+//------------------------------------------------
+// CSS3 Marquee Properties
+
+enum EMarqueeBehavior { MNONE, MSCROLL, MSLIDE, MALTERNATE, MUNFURL };
+enum EMarqueeDirection { MAUTO = 0, MLEFT = 1, MRIGHT = -1, MUP = 2, MDOWN = -2, MFORWARD = 3, MBACKWARD = -3 };
+
+class StyleMarqueeData : public Shared<StyleMarqueeData>
+{
+public:
+    StyleMarqueeData();
+    StyleMarqueeData(const StyleMarqueeData& o);
+
+    bool operator==(const StyleMarqueeData& o) const;
+    bool operator!=(const StyleMarqueeData& o) const {
+        return !(*this == o);
+    }
+
+    Length increment;
+    int speed;
+
+    int loops; // -1 means infinite.
+
+    EMarqueeBehavior behavior : 3;
+    EMarqueeDirection direction : 3;
+};
+
+// This struct is for rarely used non-inherited CSS3 properties.  By grouping them together,
+// we save space, and only allocate this object when someone actually uses
+// a non-inherited CSS3 property.
+class StyleCSS3NonInheritedData : public Shared<StyleCSS3NonInheritedData>
+{
+public:
+    StyleCSS3NonInheritedData();
+    ~StyleCSS3NonInheritedData() {}
+    StyleCSS3NonInheritedData(const StyleCSS3NonInheritedData& o);
+
+    bool operator==(const StyleCSS3NonInheritedData& o) const;
+    bool operator!=(const StyleCSS3NonInheritedData &o) const {
+        return !(*this == o);
+    }
+
+#ifdef APPLE_CHANGES	// ### we don't have those (yet)
+    float opacity;         // Whether or not we're transparent.
+    DataRef<StyleFlexibleBoxData> flexibleBox; // Flexible box properties
+#endif
+    DataRef<StyleMarqueeData> marquee; // Marquee properties
+};
 
 //------------------------------------------------
 // Inherited attributes.
@@ -380,7 +467,7 @@ enum EWhiteSpace {
 };
 
 enum ETextAlign {
-    TAAUTO, LEFT, RIGHT, CENTER, JUSTIFY, KONQ_CENTER
+    TAAUTO, LEFT, RIGHT, CENTER, JUSTIFY, KHTML_LEFT, KHTML_RIGHT, KHTML_CENTER
 };
 
 enum ETextTransform {
@@ -395,6 +482,10 @@ enum ETextDecoration {
     TDNONE = 0x0 , UNDERLINE = 0x1, OVERLINE = 0x2, LINE_THROUGH= 0x4, BLINK = 0x8
 };
 
+enum EPageBreak {
+    PBAUTO, PBALWAYS, PBAVOID
+};
+
 class StyleInheritedData : public Shared<StyleInheritedData>
 {
     StyleInheritedData& operator=(const StyleInheritedData&);
@@ -421,6 +512,11 @@ public:
 
     short border_hspacing;
     short border_vspacing;
+
+    // Paged media properties.
+    short widows;
+    short orphans;
+    EPageBreak page_break_inside : 2;
 };
 
 
@@ -430,7 +526,7 @@ enum EEmptyCell {
 
 enum ECaptionSide
 {
-    CAPTOP, CAPBOTTOM
+    CAPTOP, CAPBOTTOM, CAPLEFT, CAPRIGHT
 };
 
 
@@ -507,7 +603,7 @@ protected:
 
     // inherit
     struct InheritedFlags {
-    // 32 bit inherited, don't add to the struct, or the operator will break.
+    // 64 bit inherited, don't add to the struct, or the operator will break.
 	bool operator==( const InheritedFlags &other ) const
         {    return _iflags ==other._iflags;    }
 	bool operator!=( const InheritedFlags &other ) const
@@ -516,12 +612,12 @@ protected:
         union {
             struct {
                 EEmptyCell _empty_cells : 1 ;
-                ECaptionSide _caption_side : 1;
+                ECaptionSide _caption_side : 2;
                 EListStyleType _list_style_type : 5 ;
                 EListStylePosition _list_style_position :1;
 
                 EVisibility _visibility : 2;
-                ETextAlign _text_align : 3;
+                ETextAlign _text_align : 4;
                 ETextTransform _text_transform : 2;
                 unsigned _text_decorations : 4;
                 ECursor _cursor_style : 5;
@@ -533,14 +629,15 @@ protected:
                 bool _visuallyOrdered : 1;
                 bool _htmlHacks :1;
                 EUserInput _user_input : 2;
+                unsigned int unused : 30;
             } f;
-            Q_UINT32 _iflags;
+            Q_UINT64 _iflags;
         };
     } inherited_flags;
 
 // don't inherit
     struct NonInheritedFlags {
-    // 32 bit non-inherited, don't add to the struct, or the operator will break.
+    // 64 bit non-inherited, don't add to the struct, or the operator will break.
 	bool operator==( const NonInheritedFlags &other ) const
         {   return _niflags == other._niflags;    }
 	bool operator!=( const NonInheritedFlags &other ) const
@@ -549,6 +646,7 @@ protected:
         union {
             struct {
                 EDisplay _display : 5;
+                EDisplay _originalDisplay: 5;
                 EBackgroundRepeat _bg_repeat : 2;
                 bool _bg_attachment : 1;
                 EOverflow _overflow : 4 ;
@@ -559,13 +657,18 @@ protected:
                 ETableLayout _table_layout : 1;
                 bool _flowAroundFloats :1;
 
+                EPageBreak _page_break_before : 2;
+                EPageBreak _page_break_after : 2;
+
                 PseudoId _styleType : 3;
 		bool _affectedByHover : 1;
 		bool _affectedByActive : 1;
                 bool _hasClip : 1;
                 EUnicodeBidi _unicodeBidi : 2;
+
+                unsigned int unused : 23;
             } f;
-            Q_UINT32 _niflags;
+            Q_UINT64 _niflags;
         };
     } noninherited_flags;
 
@@ -574,6 +677,7 @@ protected:
     DataRef<StyleVisualData> visual;
     DataRef<StyleBackgroundData> background;
     DataRef<StyleSurroundData> surround;
+    DataRef<StyleCSS3NonInheritedData> css3NonInheritedData;
 
 // inherited attributes
     DataRef<StyleInheritedData> inherited;
@@ -610,8 +714,12 @@ protected:
 	inherited_flags.f._visuallyOrdered = false;
 	inherited_flags.f._htmlHacks=false;
 	inherited_flags.f._user_input = UI_NONE;
+        inherited_flags.f.unused = 0;
 
-        noninherited_flags.f._display = initialDisplay();
+	noninherited_flags._niflags = 0L; // for safety: without this, the equality method sometimes
+	                                  // makes use of uninitialised bits according to valgrind
+
+        noninherited_flags.f._display = noninherited_flags.f._originalDisplay = initialDisplay();
 	noninherited_flags.f._bg_repeat = initialBackgroundRepeat();
 	noninherited_flags.f._bg_attachment = initialBackgroundAttachment();
 	noninherited_flags.f._overflow = initialOverflow();
@@ -621,11 +729,14 @@ protected:
 	noninherited_flags.f._floating = initialFloating();
 	noninherited_flags.f._table_layout = initialTableLayout();
 	noninherited_flags.f._flowAroundFloats= initialFlowAroundFloats();
+        noninherited_flags.f._page_break_before = initialPageBreak();
+        noninherited_flags.f._page_break_after = initialPageBreak();
 	noninherited_flags.f._styleType = NOPSEUDO;
 	noninherited_flags.f._affectedByHover = false;
 	noninherited_flags.f._affectedByActive = false;
 	noninherited_flags.f._hasClip = false;
 	noninherited_flags.f._unicodeBidi = initialUnicodeBidi();
+        noninherited_flags.f.unused = 0;
     }
 
 public:
@@ -661,6 +772,7 @@ public:
 // attribute getter methods
 
     EDisplay 	display() const { return noninherited_flags.f._display; }
+    EDisplay    originalDisplay() const { return noninherited_flags.f._originalDisplay; }
 
     Length  	left() const {  return surround->offset.left; }
     Length  	right() const {  return surround->offset.right; }
@@ -677,22 +789,31 @@ public:
     Length  	minHeight() const { return box->min_height; }
     Length  	maxHeight() const { return box->max_height; }
 
+    const BorderValue& borderLeft() const { return surround->border.left; }
+    const BorderValue& borderRight() const { return surround->border.right; }
+    const BorderValue& borderTop() const { return surround->border.top; }
+    const BorderValue& borderBottom() const { return surround->border.bottom; }
+
     unsigned short  borderLeftWidth() const
     { if( surround->border.left.style == BNONE) return 0; return surround->border.left.width; }
     EBorderStyle    borderLeftStyle() const { return surround->border.left.style; }
     const QColor &  borderLeftColor() const { return surround->border.left.color; }
+    bool borderLeftIsTransparent() const { return surround->border.left.isTransparent(); }
     unsigned short  borderRightWidth() const
     { if (surround->border.right.style == BNONE) return 0; return surround->border.right.width; }
     EBorderStyle    borderRightStyle() const {  return surround->border.right.style; }
     const QColor &  	    borderRightColor() const {  return surround->border.right.color; }
+    bool borderRightIsTransparent() const { return surround->border.right.isTransparent(); }
     unsigned short  borderTopWidth() const
     { if(surround->border.top.style == BNONE) return 0; return surround->border.top.width; }
     EBorderStyle    borderTopStyle() const {return surround->border.top.style; }
     const QColor &  borderTopColor() const {  return surround->border.top.color; }
+    bool borderTopIsTransparent() const { return surround->border.top.isTransparent(); }
     unsigned short  borderBottomWidth() const
     { if(surround->border.bottom.style == BNONE) return 0; return surround->border.bottom.width; }
     EBorderStyle    borderBottomStyle() const {  return surround->border.bottom.style; }
     const QColor &  	    borderBottomColor() const {  return surround->border.bottom.color; }
+    bool borderBottomIsTransparent() const { return surround->border.bottom.isTransparent(); }
 
     unsigned short  outlineWidth() const
     { if(background->outline.style == BNONE) return 0; return background->outline.width; }
@@ -772,11 +893,27 @@ public:
 
     ECursor cursor() const { return inherited_flags.f._cursor_style; }
 
+    short widows() const { return inherited->widows; }
+    short orphans() const { return inherited->orphans; }
+    EPageBreak pageBreakInside() const { return inherited->page_break_inside; }
+    EPageBreak pageBreakBefore() const { return noninherited_flags.f._page_break_before; }
+    EPageBreak pageBreakAfter() const { return noninherited_flags.f._page_break_after; }
+
+    // CSS3 Getter Methods
+    EBoxSizing boxSizing() const { return box->box_sizing; }
     EUserInput userInput() const { return inherited_flags.f._user_input; }
 
+    Length marqueeIncrement() { return css3NonInheritedData->marquee->increment; }
+    int marqueeSpeed() { return css3NonInheritedData->marquee->speed; }
+    int marqueeLoopCount() { return css3NonInheritedData->marquee->loops; }
+    EMarqueeBehavior marqueeBehavior() { return css3NonInheritedData->marquee->behavior; }
+    EMarqueeDirection marqueeDirection() { return css3NonInheritedData->marquee->direction; }
+    // End CSS3 Getters
+
 // attribute setter methods
 
     void setDisplay(EDisplay v) {  noninherited_flags.f._display = v; }
+    void setOriginalDisplay(EDisplay v) {  noninherited_flags.f._originalDisplay = v; }
     void setPosition(EPosition v) {  noninherited_flags.f._position = v; }
     void setFloating(EFloat v) {  noninherited_flags.f._floating = v; }
 
@@ -892,8 +1029,6 @@ public:
 
     void setCursor( ECursor c ) { inherited_flags.f._cursor_style = c; }
 
-    void setUserInput(EUserInput ui) { inherited_flags.f._user_input = ui; }
-
     bool htmlHacks() const { return inherited_flags.f._htmlHacks; }
     void setHtmlHacks(bool b=true) { inherited_flags.f._htmlHacks = b; }
 
@@ -905,6 +1040,23 @@ public:
     bool hasAutoZIndex() const { return box->z_auto; }
     void setHasAutoZIndex() { SET_VAR(box, z_auto, true ); }
 
+    void setWidows(short w) { SET_VAR(inherited, widows, w); }
+    void setOrphans(short o) { SET_VAR(inherited, orphans, o); }
+    void setPageBreakInside(EPageBreak b) { SET_VAR(inherited, page_break_inside, b); }
+    void setPageBreakBefore(EPageBreak b) { noninherited_flags.f._page_break_before = b; }
+    void setPageBreakAfter(EPageBreak b) { noninherited_flags.f._page_break_after = b; }
+
+    // CSS3 Setters
+    void setBoxSizing( EBoxSizing b ) { SET_VAR(box,box_sizing,b); }
+    void setUserInput(EUserInput ui) { inherited_flags.f._user_input = ui; }
+
+    void setMarqueeIncrement(const Length& f) { SET_VAR(css3NonInheritedData.access()->marquee, increment, f); }
+    void setMarqueeSpeed(int f) { SET_VAR(css3NonInheritedData.access()->marquee, speed, f); }
+    void setMarqueeDirection(EMarqueeDirection d) { SET_VAR(css3NonInheritedData.access()->marquee, direction, d); }
+    void setMarqueeBehavior(EMarqueeBehavior b) { SET_VAR(css3NonInheritedData.access()->marquee, behavior, b); }
+    void setMarqueeLoopCount(int i) { SET_VAR(css3NonInheritedData.access()->marquee, loops, i); }
+    // End CSS3 Setters
+
     QPalette palette() const { return visual->palette; }
     void setPaletteColor(QPalette::ColorGroup g, QColorGroup::ColorRole r, const QColor& c);
     void resetPalette() // Called when the desktop color scheme changes.
@@ -923,6 +1075,18 @@ public:
     enum Diff { Equal, NonVisible = Equal, Visible, Position, Layout, CbLayout };
     Diff diff( const RenderStyle *other ) const;
 
+    bool isDisplayReplacedType() {
+        return display() == INLINE_BLOCK ||/* display() == INLINE_BOX ||*/ display() == INLINE_TABLE;
+    }
+    bool isDisplayInlineType() {
+        return display() == INLINE || isDisplayReplacedType();
+    }
+    bool isOriginalDisplayInlineType() {
+        return originalDisplay() == INLINE || originalDisplay() == INLINE_BLOCK ||
+               /*originalDisplay() == INLINE_BOX ||*/ originalDisplay() == INLINE_TABLE;
+    }
+
+
 #ifdef ENABLE_DUMP
     QString createDiff( const RenderStyle &parent ) const;
 #endif
@@ -941,9 +1105,11 @@ public:
     static EListStylePosition initialListStylePosition() { return OUTSIDE; }
     static EListStyleType initialListStyleType() { return DISC; }
     static EOverflow initialOverflow() { return OVISIBLE; }
+    static EPageBreak initialPageBreak() { return PBAUTO; }
     static EPosition initialPosition() { return STATIC; }
     static ETableLayout initialTableLayout() { return TAUTO; }
     static EUnicodeBidi initialUnicodeBidi() { return UBNormal; }
+    static EBoxSizing initialBoxSizing() { return CONTENT_BOX; }
     static ETextTransform initialTextTransform() { return TTNONE; }
     static EVisibility initialVisibility() { return VISIBLE; }
     static EWhiteSpace initialWhiteSpace() { return NORMAL; }
@@ -973,6 +1139,11 @@ public:
     static bool initialFlowAroundFloats() { return false; }
     static int initialOutlineOffset() { return 0; }
     static float initialOpacity() { return 1.0f; }
+    static int initialMarqueeLoopCount() { return -1; }
+    static int initialMarqueeSpeed() { return 85; }
+    static Length initialMarqueeIncrement() { return Length(6, Fixed); }
+    static EMarqueeBehavior initialMarqueeBehavior() { return MSCROLL; }
+    static EMarqueeDirection initialMarqueeDirection() { return MAUTO; }
 };
 
 
