@@ -20,7 +20,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_nodeimpl.h,v 1.151.2.2 2004/05/07 11:29:15 savernik Exp $
  */
 #ifndef _DOM_NodeImpl_h_
 #define _DOM_NodeImpl_h_
@@ -121,24 +120,6 @@ public:
     virtual void setFirstChild(NodeImpl *child);
     virtual void setLastChild(NodeImpl *child);
 
-    /** (Not part of the official DOM)
-     * Returns the next leaf node.
-     *
-     * Using this function delivers leaf nodes as if the whole DOM tree
-     * were a linear chain of its leaf nodes.
-     * @return next leaf node or 0 if there are no more.
-     */
-    NodeImpl *nextLeafNode() const;
-
-    /** (Not part of the official DOM)
-     * Returns the previous leaf node.
-     *
-     * Using this function delivers leaf nodes as if the whole DOM tree
-     * were a linear chain of its leaf nodes.
-     * @return previous leaf node or 0 if there are no more.
-     */
-    NodeImpl *prevLeafNode() const;
-
     // used by the parser. Doesn't do as many error checkings as
     // appendChild(), and returns the node into which will be parsed next.
     virtual NodeImpl *addChild(NodeImpl *newChild);
@@ -178,11 +159,12 @@ public:
     {
         MouseEvent( int _button, MouseEventType _type,
                     const DOMString &_url = DOMString(), const DOMString& _target = DOMString(),
-                    NodeImpl *_innerNode = 0)
+                    NodeImpl *_innerNode = 0, NodeImpl *_innerNonSharedNode = 0)
             {
                 button = _button; type = _type;
                 url = _url; target = _target;
                 innerNode = _innerNode;
+		innerNonSharedNode = _innerNonSharedNode;
             }
 
         int button;
@@ -190,6 +172,7 @@ public:
         DOMString url; // url under mouse or empty
         DOMString target;
         Node innerNode;
+	Node innerNonSharedNode;
     };
 
     // for LINK and STYLE
@@ -219,14 +202,11 @@ public:
     unsigned short tabIndex() const { return m_tabIndex; }
     void setTabIndex(unsigned short _tabIndex) { m_tabIndex = _tabIndex; }
 
-    /**
-     * whether this node can receive the keyboard focus.
-     */
-    virtual bool isSelectable() const { return false; }
+    virtual bool isFocusable() const { return false; }
+    virtual bool isMouseFocusable() const { return isFocusable(); }
+    virtual bool isTabFocusable() const { return isFocusable(); }
 
     virtual bool isInline() const;
-    virtual QString toHTML() const;
-    QString recursive_toHTML(bool start = false) const;
 
     virtual void getCaret(int offset, bool override, int &_x, int &_y, int &width, int &height);
     virtual QRect getRect() const;
@@ -294,6 +274,7 @@ public:
 
     khtml::RenderObject *renderer() const { return m_render; }
     khtml::RenderObject *nextRenderer();
+    void setRenderer(khtml::RenderObject* renderer) { m_render = renderer; }
 
     void checkSetPrefix(const DOMString &_prefix, int &exceptioncode);
     void checkAddChild(NodeImpl *newChild, int &exceptioncode);
@@ -310,8 +291,8 @@ public:
     /**
      * Returns the maximum caret offset that is allowed for this node.
      *
-     * This default implementation always returns 1. Textual child nodes
-     * return the character count instead.
+     * This default implementation always returns the node count.
+     * Textual child nodes return the character count instead.
      */
     virtual long maxOffset() const;
 
@@ -389,6 +370,8 @@ public:
      */
     virtual void childrenChanged();
 
+    virtual DOMString toString() const = 0;
+
 private: // members
     DocumentPtr *document;
     NodeImpl *m_previous;
@@ -445,6 +428,7 @@ public:
     virtual void attach();
     virtual void detach();
 
+
     bool getUpperLeftCorner(int &xPos, int &yPos) const;
     bool getLowerRightCorner(int &xPos, int &yPos) const;
 
