@@ -1,3 +1,4 @@
+// -*- mode: c++; c-basic-offset: 2 -*-
 /* This file is part of the KDE project
    Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>
              (C) 1999 David Faure <faure@kde.org>
@@ -46,10 +47,31 @@ public:
     m_bAllowNestedParts = false;
     m_bIgnoreScrollBars = false;
     m_activationButtonMask = Qt::LeftButton | Qt::MidButton | Qt::RightButton;
+    m_reason = PartManager::NoReason;
   }
   ~PartManagerPrivate()
   {
   }
+  void setReason( QEvent* ev ) {
+    switch( ev->type() ) {
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonDblClick: {
+      QMouseEvent* mev = static_cast<QMouseEvent *>( ev );
+      m_reason = mev->button() == Qt::LeftButton
+                 ? PartManager::ReasonLeftClick
+                 : ( mev->button() == Qt::MidButton
+                     ? PartManager::ReasonMidClick
+                     : PartManager::ReasonRightClick );
+      break;
+    }
+    case QEvent::FocusIn:
+      m_reason = static_cast<QFocusEvent *>( ev )->reason();
+      break;
+    default:
+      kdWarning(1000) << "PartManagerPrivate::setReason got unexpected ev type " << ev->type() << endl;
+      break;
+    }
+  }
 
   Part * m_activePart;
   QWidget *m_activeWidget;
@@ -65,6 +87,7 @@ public:
   short int m_activationButtonMask;
   bool m_bIgnoreScrollBars;
   bool m_bAllowNestedParts;
+  int m_reason;
 };
 
 }
@@ -101,7 +124,9 @@ PartManager::~PartManager()
                 this, SLOT( slotManagedTopLevelWidgetDestroyed() ) );
 
   for ( QPtrListIterator<Part> it( d->m_parts ); it.current(); ++it )
+  {
       it.current()->setManager( 0 );
+  }
 
   // core dumps ... setActivePart( 0L );
   qApp->removeEventFilter( this );
@@ -170,7 +195,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
   {
       mev = static_cast<QMouseEvent *>( ev );
 #ifdef DEBUG_PARTMANAGER
-      kdDebug() << "PartManager::eventFilter button: " << mev->button() << " " << "d->m_activationButtonMask=" << d->m_activationButtonMask << endl;
+      kdDebug(1000) << "PartManager::eventFilter button: " << mev->button() << " " << "d->m_activationButtonMask=" << d->m_activationButtonMask << endl;
 #endif
       if ( ( mev->button() & d->m_activationButtonMask ) == 0 )
         return false; // ignore this button
@@ -211,7 +236,9 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
 #ifdef DEBUG_PARTMANAGER
           kdDebug(1000) << "PartManager::eventFilter dblclick -> setActivePart" << part << endl;
 #endif
+          d->setReason( ev );
           setActivePart( part, w );
+          d->m_reason = NoReason;
           return true;
         }
 
@@ -224,7 +251,9 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
 #ifdef DEBUG_PARTMANAGER
               kdDebug(1000) << "Part " << part << " (non-selectable) made active because " << w->className() << " got event" << " " << evType << endl;
 #endif
+              d->setReason( ev );
               setActivePart( part, w );
+              d->m_reason = NoReason;
           }
           return true;
         }
@@ -233,7 +262,9 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
 #ifdef DEBUG_PARTMANAGER
           kdDebug(1000) << "Part " << part << " made active (from selected) because " << w->className() << " got event" << " " << evType << endl;
 #endif
+          d->setReason( ev );
           setActivePart( part, w );
+          d->m_reason = NoReason;
           return true;
         }
         else if ( d->m_activeWidget == w && d->m_activePart == part )
@@ -249,7 +280,9 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
 #ifdef DEBUG_PARTMANAGER
         kdDebug(1000) << "Part " << part << " made active because " << w->className() << " got event" << " " << evType << endl;
 #endif
+        d->setReason( ev );
         setActivePart( part, w );
+        d->m_reason = NoReason;
       }
 
       return false;
@@ -299,8 +332,13 @@ Part * PartManager::findPartFromWidget( QWidget * widget )
 
 void PartManager::addPart( Part *part, bool setActive )
 {
-    if ( d->m_parts.findRef( part ) != -1 ) // don't add parts more than once :)
-      return;
+  if ( d->m_parts.findRef( part ) != -1 ) // don't add parts more than once :)
+  {
+#ifdef DEBUG_PARTMANAGER
+    kdWarning(1000) << k_funcinfo << part << " already added" << kdBacktrace(5) << endl;
+#endif
+    return;
+  }
 
   d->m_parts.append( part );
 
@@ -338,7 +376,10 @@ void PartManager::removePart( Part *part )
 
   //Warning. The part could be already deleted
   //kdDebug(1000) << QString("Part %1 removed").arg(part->name()) << endl;
-  d->m_parts.removeRef( part );
+  int nb = d->m_parts.count();
+  bool ok = d->m_parts.removeRef( part );
+  Q_ASSERT( ok );
+  Q_ASSERT( (int)d->m_parts.count() == nb-1 );
   part->setManager(0);
 
   emit partRemoved( part );
@@ -388,8 +429,10 @@ void PartManager::setActivePart( Part *part, QWidget *widget )
     }
   }
 
-  //kdDebug() << "PartManager::setActivePart d->m_activePart=" << d->m_activePart << "<->part=" << part
-  //          << " d->m_activeWidget=" << d->m_activeWidget << "<->widget=" << widget << endl;
+#ifdef DEBUG_PARTMANAGER
+  kdDebug(1000) << "PartManager::setActivePart d->m_activePart=" << d->m_activePart << "<->part=" << part
+                << " d->m_activeWidget=" << d->m_activeWidget << "<->widget=" << widget << endl;
+#endif
 
   // don't activate twice
   if ( d->m_activePart && part && d->m_activePart == part &&
@@ -547,6 +590,11 @@ void PartManager::slotManagedTopLevelWidgetDestroyed()
   removeManagedTopLevelWidget( widget );
 }
 
+int PartManager::reason() const
+{
+  return d->m_reason;
+}
+
 void PartManager::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
 
